@@ -8,22 +8,21 @@ from .externals.six.moves import configparser
 import fnmatch
 from glob import glob, iglob
 import os
+import stat
+import sys
 import re
 import shutil
 from warnings import warn
 
 import numpy as np
 from numpy import dot
-from scipy.optimize import leastsq
-from scipy.spatial.distance import cdist
-from scipy.linalg import norm
 
 from .io.meas_info import read_fiducials, write_fiducials
 from .label import read_label, Label
 from .source_space import (add_source_space_distances, read_source_spaces,
                            write_source_spaces)
-from .surface import (read_surface, write_surface, read_bem_surfaces,
-                      write_bem_surface)
+from .surface import read_surface, write_surface
+from .bem import read_bem_surfaces, write_bem_surfaces
 from .transforms import rotation, rotation3d, scaling, translation
 from .utils import get_config, get_subjects_dir, logger, pformat
 from functools import reduce
@@ -42,6 +41,19 @@ fid_fname_general = os.path.join(bem_dirname, "{head}-fiducials.fif")
 src_fname = os.path.join(bem_dirname, '{subject}-{spacing}-src.fif')
 
 
+def _make_writable(fname):
+    os.chmod(fname, stat.S_IMODE(os.lstat(fname)[stat.ST_MODE]) | 128)  # write
+
+
+def _make_writable_recursive(path):
+    """Recursively set writable"""
+    if sys.platform.startswith('win'):
+        return  # can't safely set perms
+    for root, dirs, files in os.walk(path, topdown=False):
+        for f in dirs + files:
+            _make_writable(os.path.join(root, f))
+
+
 def create_default_subject(mne_root=None, fs_home=None, update=False,
                            subjects_dir=None):
     """Create an average brain subject for subjects without structural MRI
@@ -92,44 +104,41 @@ def create_default_subject(mne_root=None, fs_home=None, update=False,
     if fs_home is None:
         fs_home = get_config('FREESURFER_HOME', fs_home)
         if fs_home is None:
-            err = ("FREESURFER_HOME environment variable not found. Please "
-                   "specify the fs_home parameter in your call to "
-                   "create_default_subject().")
-            raise ValueError(err)
+            raise ValueError(
+                "FREESURFER_HOME environment variable not found. Please "
+                "specify the fs_home parameter in your call to "
+                "create_default_subject().")
     if mne_root is None:
         mne_root = get_config('MNE_ROOT', mne_root)
         if mne_root is None:
-            err = ("MNE_ROOT environment variable not found. Please "
-                   "specify the mne_root parameter in your call to "
-                   "create_default_subject().")
-            raise ValueError(err)
+            raise ValueError("MNE_ROOT environment variable not found. Please "
+                             "specify the mne_root parameter in your call to "
+                             "create_default_subject().")
 
     # make sure freesurfer files exist
     fs_src = os.path.join(fs_home, 'subjects', 'fsaverage')
     if not os.path.exists(fs_src):
-        err = ('fsaverage not found at %r. Is fs_home specified '
-               'correctly?' % fs_src)
-        raise IOError(err)
+        raise IOError('fsaverage not found at %r. Is fs_home specified '
+                      'correctly?' % fs_src)
     for name in ('label', 'mri', 'surf'):
         dirname = os.path.join(fs_src, name)
         if not os.path.isdir(dirname):
-            err = ("Freesurfer fsaverage seems to be incomplete: No directory "
-                   "named %s found in %s" % (name, fs_src))
-            raise IOError(err)
+            raise IOError("Freesurfer fsaverage seems to be incomplete: No "
+                          "directory named %s found in %s" % (name, fs_src))
 
     # make sure destination does not already exist
     dest = os.path.join(subjects_dir, 'fsaverage')
     if dest == fs_src:
-        err = ("Your subjects_dir points to the freesurfer subjects_dir (%r). "
-               "The default subject can not be created in the freesurfer "
-               "installation directory; please specify a different "
-               "subjects_dir." % subjects_dir)
-        raise IOError(err)
+        raise IOError(
+            "Your subjects_dir points to the freesurfer subjects_dir (%r). "
+            "The default subject can not be created in the freesurfer "
+            "installation directory; please specify a different "
+            "subjects_dir." % subjects_dir)
     elif (not update) and os.path.exists(dest):
-        err = ("Can not create fsaverage because %r already exists in "
-               "subjects_dir %r. Delete or rename the existing fsaverage "
-               "subject folder." % ('fsaverage', subjects_dir))
-        raise IOError(err)
+        raise IOError(
+            "Can not create fsaverage because %r already exists in "
+            "subjects_dir %r. Delete or rename the existing fsaverage "
+            "subject folder." % ('fsaverage', subjects_dir))
 
     # make sure mne files exist
     mne_fname = os.path.join(mne_root, 'share', 'mne', 'mne_analyze',
@@ -138,14 +147,14 @@ def create_default_subject(mne_root=None, fs_home=None, update=False,
     for name in mne_files:
         fname = mne_fname % name
         if not os.path.isfile(fname):
-            err = ("MNE fsaverage incomplete: %s file not found at "
-                   "%s" % (name, fname))
-            raise IOError(err)
+            raise IOError("MNE fsaverage incomplete: %s file not found at "
+                          "%s" % (name, fname))
 
     # copy fsaverage from freesurfer
     logger.info("Copying fsaverage subject from freesurfer directory...")
     if (not update) or not os.path.exists(dest):
         shutil.copytree(fs_src, dest)
+        _make_writable_recursive(dest)
 
     # add files from mne
     dest_bem = os.path.join(dest, 'bem')
@@ -153,6 +162,7 @@ def create_default_subject(mne_root=None, fs_home=None, update=False,
         os.mkdir(dest_bem)
     logger.info("Copying auxiliary fsaverage files from mne directory...")
     dest_fname = os.path.join(dest_bem, 'fsaverage-%s.fif')
+    _make_writable_recursive(dest_bem)
     for name in mne_files:
         if not os.path.exists(dest_fname % name):
             shutil.copy(mne_fname % name, dest_bem)
@@ -167,7 +177,7 @@ def _decimate_points(pts, res=10):
 
     Parameters
     ----------
-    pts : array, shape = (n_points, 3)
+    pts : array, shape (n_points, 3)
         The points making up the head shape.
     res : scalar
         The resolution of the voxel space (side length of each voxel).
@@ -177,6 +187,7 @@ def _decimate_points(pts, res=10):
     pts : array, shape = (n_points, 3)
         The decimated points.
     """
+    from scipy.spatial.distance import cdist
     pts = np.asarray(pts)
 
     # find the bin edges for the voxel space
@@ -297,12 +308,12 @@ def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
         A single tuple containing the translation, rotation and scaling
         parameters in that order.
     """
+    from scipy.optimize import leastsq
     src_pts = np.atleast_2d(src_pts)
     tgt_pts = np.atleast_2d(tgt_pts)
     if src_pts.shape != tgt_pts.shape:
-        err = ("src_pts and tgt_pts must have same shape "
-               "(got {0}, {1})".format(src_pts.shape, tgt_pts.shape))
-        raise ValueError(err)
+        raise ValueError("src_pts and tgt_pts must have same shape (got "
+                         "{0}, {1})".format(src_pts.shape, tgt_pts.shape))
 
     rotate = bool(rotate)
     translate = bool(translate)
@@ -345,9 +356,9 @@ def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
         if x0 is None:
             x0 = (0, 0, 0, 0, 0, 0, 1)
     else:
-        err = ("The specified parameter combination is not implemented: "
-               "rotate=%r, translate=%r, scale=%r" % param_info)
-        raise NotImplementedError(err)
+        raise NotImplementedError(
+            "The specified parameter combination is not implemented: "
+            "rotate=%r, translate=%r, scale=%r" % param_info)
 
     x, _, _, _, _ = leastsq(error, x0, full_output=True)
 
@@ -369,62 +380,8 @@ def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
     elif out == 'trans':
         return trans
     else:
-        err = ("Invalid out parameter: %r. Needs to be 'params' or "
-               "'trans'." % out)
-        raise ValueError(err)
-
-
-def get_ras_to_neuromag_trans(nasion, lpa, rpa):
-    """Construct a transformation matrix to the MNE head coordinate system
-
-    Construct a transformation matrix from an arbitrary RAS coordinate system
-    to the MNE head coordinate system, in which the x axis passes through the
-    two preauricular points, and the y axis passes through the nasion and is
-    normal to the x axis. (see mne manual, pg. 97)
-
-    Parameters
-    ----------
-    nasion : array_like, shape = (3,)
-        Nasion point coordinate.
-    lpa : array_like, shape = (3,)
-        Left peri-auricular point coordinate.
-    rpa : array_like, shape = (3,)
-        Right peri-auricular point coordinate.
-
-    Returns
-    -------
-    trans : numpy.array, shape = (4, 4)
-        Transformation matrix to MNE head space.
-    """
-    # check input args
-    nasion = np.asarray(nasion)
-    lpa = np.asarray(lpa)
-    rpa = np.asarray(rpa)
-    for pt in (nasion, lpa, rpa):
-        if pt.ndim != 1 or len(pt) != 3:
-            err = ("Points have to be provided as one dimensional arrays of "
-                   "length 3.")
-            raise ValueError(err)
-
-    right = rpa - lpa
-    right_unit = right / norm(right)
-
-    origin = lpa + np.dot(nasion - lpa, right_unit) * right_unit
-
-    anterior = nasion - origin
-    anterior_unit = anterior / norm(anterior)
-
-    superior_unit = np.cross(right_unit, anterior_unit)
-
-    x, y, z = -origin
-    origin_trans = translation(x, y, z)
-
-    trans_l = np.vstack((right_unit, anterior_unit, superior_unit, [0, 0, 0]))
-    trans_r = np.reshape([0, 0, 0, 1], (4, 1))
-    rot_trans = np.hstack((trans_l, trans_r))
-
-    trans = np.dot(rot_trans, origin_trans)
-    return trans
+        raise ValueError("Invalid out parameter: %r. Needs to be 'params' or "
+                         "'trans'." % out)
 
 
 def _point_cloud_error(src_pts, tgt_pts):
@@ -443,6 +400,7 @@ def _point_cloud_error(src_pts, tgt_pts):
         For each point in ``src_pts``, the distance to the closest point in
         ``tgt_pts``.
     """
+    from scipy.spatial.distance import cdist
     Y = cdist(src_pts, tgt_pts, 'euclidean')
     dist = Y.min(axis=1)
     return dist
@@ -513,6 +471,7 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
     the distance of each src_pt to the closest tgt_pt can be used as an
     estimate of the distance of src_pt to tgt_pts.
     """
+    from scipy.optimize import leastsq
     kwargs = {'epsfcn': 0.01}
     kwargs.update(leastsq_args)
 
@@ -539,6 +498,7 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
     param_info = (rotate, translate, scale)
     if param_info == (True, False, 0):
         x0 = x0 or (0, 0, 0)
+
         def error(x):
             rx, ry, rz = x
             trans = rotation3d(rx, ry, rz)
@@ -547,6 +507,7 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
             return err
     elif param_info == (True, False, 1):
         x0 = x0 or (0, 0, 0, 1)
+
         def error(x):
             rx, ry, rz, s = x
             trans = rotation3d(rx, ry, rz) * s
@@ -555,6 +516,7 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
             return err
     elif param_info == (True, False, 3):
         x0 = x0 or (0, 0, 0, 1, 1, 1)
+
         def error(x):
             rx, ry, rz, sx, sy, sz = x
             trans = rotation3d(rx, ry, rz) * [sx, sy, sz]
@@ -563,6 +525,7 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
             return err
     elif param_info == (True, True, 0):
         x0 = x0 or (0, 0, 0, 0, 0, 0)
+
         def error(x):
             rx, ry, rz, tx, ty, tz = x
             trans = dot(translation(tx, ty, tz), rotation(rx, ry, rz))
@@ -570,9 +533,9 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
             err = errfunc(est[:, :3], tgt_pts)
             return err
     else:
-        err = ("The specified parameter combination is not implemented: "
-               "rotate=%r, translate=%r, scale=%r" % param_info)
-        raise NotImplementedError(err)
+        raise NotImplementedError(
+            "The specified parameter combination is not implemented: "
+            "rotate=%r, translate=%r, scale=%r" % param_info)
 
     est, _, info, msg, _ = leastsq(error, x0, full_output=True, **kwargs)
     logger.debug("fit_point_cloud leastsq (%i calls) info: %s", info['nfev'],
@@ -583,9 +546,8 @@ def fit_point_cloud(src_pts, tgt_pts, rotate=True, translate=True,
     elif out == 'trans':
         return _trans_from_params(param_info, est)
     else:
-        err = ("Invalid out parameter: %r. Needs to be 'params' or "
-               "'trans'." % out)
-        raise ValueError(err)
+        raise ValueError("Invalid out parameter: %r. Needs to be 'params' or "
+                         "'trans'." % out)
 
 
 def _find_label_paths(subject='fsaverage', pattern=None, subjects_dir=None):
@@ -651,12 +613,11 @@ def _find_mri_paths(subject='fsaverage', subjects_dir=None):
     # surf/ files
     paths['surf'] = surf = []
     surf_fname = os.path.join(surf_dirname, '{name}')
-    surf_names = ('orig', 'orig_avg',
-                  'inflated', 'inflated_avg', 'inflated_pre',
-                  'pial', 'pial_avg',
-                  'smoothwm',
-                  'white', 'white_avg',
-                  'sphere', 'sphere.reg', 'sphere.reg.avg')
+    surf_names = ('inflated', 'sphere', 'sphere.reg', 'white')
+    if os.getenv('_MNE_FEW_SURFACES', '') != 'true':  # for testing
+        surf_names = surf_names + (
+            'orig', 'orig_avg', 'inflated_avg', 'inflated_pre', 'pial',
+            'pial_avg', 'smoothwm', 'white_avg', 'sphere.reg.avg')
     for name in surf_names:
         for hemi in ('lh.', 'rh.'):
             fname = pformat(surf_fname, name=hemi + name)
@@ -754,31 +715,6 @@ def _mri_subject_has_bem(subject, subjects_dir=None):
     return bool(len(fnames))
 
 
-def read_elp(fname):
-    """Read point coordinates from a text file
-
-    Parameters
-    ----------
-    fname : str
-        Absolute path to laser point file (*.txt).
-
-    Returns
-    -------
-    elp_points : array, [n_points x 3]
-        Point coordinates.
-    """
-    pattern = re.compile(r'(\-?\d+\.\d+)\s+(\-?\d+\.\d+)\s+(\-?\d+\.\d+)')
-    with open(fname) as fid:
-        elp_points = pattern.findall(fid.read())
-    elp_points = np.array(elp_points, dtype=float)
-    if elp_points.shape[1] != 3:
-        err = ("File %r does not contain 3 columns as required; got shape "
-               "%s." % (fname, elp_points.shape))
-        raise ValueError(err)
-
-    return elp_points
-
-
 def read_mri_cfg(subject, subjects_dir=None):
     """Read information from the cfg file of a scaled MRI brain
 
@@ -798,9 +734,8 @@ def read_mri_cfg(subject, subjects_dir=None):
     fname = os.path.join(subjects_dir, subject, 'MRI scaling parameters.cfg')
 
     if not os.path.exists(fname):
-        err = ("%r does not seem to be a scaled mri subject: %r does not "
-               "exist." % (subject, fname))
-        raise IOError(err)
+        raise IOError("%r does not seem to be a scaled mri subject: %r does "
+                      "not exist." % (subject, fname))
 
     logger.info("Reading MRI cfg file %s" % fname)
     config = configparser.RawConfigParser()
@@ -854,11 +789,10 @@ def _write_mri_config(fname, subject_from, subject_to, scale):
 
 
 def _scale_params(subject_to, subject_from, scale, subjects_dir):
-    subjects_dir = get_subjects_dir(subjects_dir, True)
+    subjects_dir = get_subjects_dir(subjects_dir, raise_error=True)
     if (subject_from is None) != (scale is None):
-        err = ("Need to provide either both subject_from and scale "
-               "parameters, or neither.")
-        raise TypeError(err)
+        raise TypeError("Need to provide either both subject_from and scale "
+                        "parameters, or neither.")
 
     if subject_from is None:
         cfg = read_mri_cfg(subject_to, subjects_dir)
@@ -872,9 +806,8 @@ def _scale_params(subject_to, subject_from, scale, subjects_dir):
         elif scale.shape == (3,):
             n_params = 3
         else:
-            err = ("Invalid shape for scale parameer. Need scalar or array of "
-                   "length 3. Got %s." % str(scale))
-            raise ValueError(err)
+            raise ValueError("Invalid shape for scale parameer. Need scalar "
+                             "or array of length 3. Got %s." % str(scale))
 
     return subjects_dir, subject_from, n_params, scale
 
@@ -914,11 +847,11 @@ def scale_bem(subject_to, bem_name, subject_from=None, scale=None,
 
     surfs = read_bem_surfaces(src)
     if len(surfs) != 1:
-        err = ("BEM file with more than one surface: %r" % src)
-        raise NotImplementedError(err)
+        raise NotImplementedError("BEM file with more than one surface: %r"
+                                  % src)
     surf0 = surfs[0]
     surf0['rr'] = surf0['rr'] * scale
-    write_bem_surface(dst, surf0)
+    write_bem_surfaces(dst, surf0)
 
 
 def scale_labels(subject_to, pattern=None, overwrite=False, subject_from=None,
@@ -932,7 +865,7 @@ def scale_labels(subject_to, pattern=None, overwrite=False, subject_from=None,
     pattern : str | None
         Pattern for finding the labels relative to the label directory in the
         MRI subject directory (e.g., "lh.BA3a.label" will scale
-        "fsaverage/label/lh.BA3a.label"; "aparc/*.label" will find all labels
+        "fsaverage/label/lh.BA3a.label"; "aparc/\*.label" will find all labels
         in the "fsaverage/label/aparc" directory). With None, scale all labels.
     overwrite : bool
         Overwrite any label file that already exists for subject_to (otherwise
@@ -1014,9 +947,8 @@ def scale_mri(subject_from, subject_to, scale, overwrite=False,
         if overwrite:
             shutil.rmtree(dest)
         else:
-            err = ("Subject directory for %s already exists: "
-                   "%r" % (subject_to, dest))
-            raise IOError(err)
+            raise IOError("Subject directory for %s already exists: %r"
+                          % (subject_to, dest))
 
     for dirname in paths['dirs']:
         dir_ = dirname.format(subject=subject_to, subjects_dir=subjects_dir)
@@ -1121,8 +1053,8 @@ def scale_source_space(subject_to, src_name, subject_from=None, scale=None,
     elif n_params == 3:
         norm_scale = 1. / scale
     else:
-        err = ("Invalid n_params entry in MRI cfg file: %s" % str(n_params))
-        raise RuntimeError(err)
+        raise RuntimeError("Invalid n_params entry in MRI cfg file: %s"
+                           % str(n_params))
 
     # read and scale the source space [in m]
     sss = read_source_spaces(src)
