@@ -10,28 +10,33 @@ import os.path as op
 import glob
 from copy import deepcopy
 import warnings
+import itertools as itt
 
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
-                           assert_allclose)
-from nose.tools import (assert_true, assert_raises, assert_equal,
-                        assert_not_equal)
+                           assert_allclose, assert_equal)
+from nose.tools import assert_true, assert_raises, assert_not_equal
 
-from mne import pick_types, pick_channels
+from mne.datasets import testing
 from mne.io.constants import FIFF
-from mne.io import (Raw, concatenate_raws,
-                    get_chpi_positions, set_eeg_reference)
-from mne import concatenate_events, find_events, equalize_channels
-from mne.utils import (_TempDir, requires_nitime, requires_pandas,
-                       requires_mne, run_subprocess)
-from mne.externals.six.moves import zip
-from mne.externals.six.moves import cPickle as pickle
+from mne.io import Raw, RawArray, concatenate_raws, read_raw_fif
+from mne.io.tests.test_raw import _test_concat
+from mne import (concatenate_events, find_events, equalize_channels,
+                 compute_proj_raw, pick_types, pick_channels, create_info)
+from mne.utils import (_TempDir, requires_pandas, slow_test,
+                       requires_mne, run_subprocess, run_tests_if_main)
+from mne.externals.six.moves import zip, cPickle as pickle
+from mne.io.proc_history import _get_sss_rank
+from mne.io.pick import _picks_by_type
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
+data_dir = op.join(testing.data_path(download=False), 'MEG', 'sample')
+fif_fname = op.join(data_dir, 'sample_audvis_trunc_raw.fif')
+
 base_dir = op.join(op.dirname(__file__), '..', '..', 'tests', 'data')
-fif_fname = op.join(base_dir, 'test_raw.fif')
-fif_gz_fname = op.join(base_dir, 'test_raw.fif.gz')
+test_fif_fname = op.join(base_dir, 'test_raw.fif')
+test_fif_gz_fname = op.join(base_dir, 'test_raw.fif.gz')
 ctf_fname = op.join(base_dir, 'test_ctf_raw.fif')
 ctf_comp_fname = op.join(base_dir, 'test_ctf_comp_raw.fif')
 fif_bad_marked_fname = op.join(base_dir, 'test_withbads_raw.fif')
@@ -40,29 +45,69 @@ bad_file_wrong = op.join(base_dir, 'test_wrong_bads.txt')
 hp_fname = op.join(base_dir, 'test_chpi_raw_hp.txt')
 hp_fif_fname = op.join(base_dir, 'test_chpi_raw_sss.fif')
 
-tempdir = _TempDir()
+
+def test_fix_types():
+    """Test fixing of channel types
+    """
+    for fname, change in ((hp_fif_fname, True), (test_fif_fname, False),
+                          (ctf_fname, False)):
+        raw = Raw(fname)
+        mag_picks = pick_types(raw.info, meg='mag')
+        other_picks = np.setdiff1d(np.arange(len(raw.ch_names)), mag_picks)
+        # we don't actually have any files suffering from this problem, so
+        # fake it
+        if change:
+            for ii in mag_picks:
+                raw.info['chs'][ii]['coil_type'] = FIFF.FIFFV_COIL_VV_MAG_T2
+        orig_types = np.array([ch['coil_type'] for ch in raw.info['chs']])
+        raw.fix_mag_coil_types()
+        new_types = np.array([ch['coil_type'] for ch in raw.info['chs']])
+        if not change:
+            assert_array_equal(orig_types, new_types)
+        else:
+            assert_array_equal(orig_types[other_picks], new_types[other_picks])
+            assert_true((orig_types[mag_picks] != new_types[mag_picks]).all())
+            assert_true((new_types[mag_picks] ==
+                         FIFF.FIFFV_COIL_VV_MAG_T3).all())
 
 
+def test_concat():
+    """Test RawFIF concatenation
+    """
+    # we trim the file to save lots of memory and some time
+    tempdir = _TempDir()
+    raw = read_raw_fif(test_fif_fname)
+    raw.crop(0, 2., copy=False)
+    test_name = op.join(tempdir, 'test_raw.fif')
+    raw.save(test_name)
+    # now run the standard test
+    _test_concat(read_raw_fif, test_name)
+
+
+@testing.requires_testing_data
 def test_hash_raw():
     """Test hashing raw objects
     """
-    raw = Raw(fif_fname)
+    raw = read_raw_fif(fif_fname)
     assert_raises(RuntimeError, raw.__hash__)
-    raw = Raw(fif_fname, preload=True).crop(0, 0.5)
-    raw_2 = Raw(fif_fname, preload=True).crop(0, 0.5)
+    raw = Raw(fif_fname).crop(0, 0.5, False)
+    raw.load_data()
+    raw_2 = Raw(fif_fname).crop(0, 0.5, False)
+    raw_2.load_data()
     assert_equal(hash(raw), hash(raw_2))
     # do NOT use assert_equal here, failing output is terrible
-    assert_true(pickle.dumps(raw) == pickle.dumps(raw_2))
+    assert_equal(pickle.dumps(raw), pickle.dumps(raw_2))
 
     raw_2._data[0, 0] -= 1
     assert_not_equal(hash(raw), hash(raw_2))
 
 
+@testing.requires_testing_data
 def test_subject_info():
     """Test reading subject information
     """
-    raw = Raw(fif_fname)
-    raw.crop(0, 1, False)
+    tempdir = _TempDir()
+    raw = Raw(fif_fname).crop(0, 1, False)
     assert_true(raw.info['subject_info'] is None)
     # fake some subject data
     keys = ['id', 'his_id', 'last_name', 'first_name', 'birthday', 'sex',
@@ -85,23 +130,7 @@ def test_subject_info():
     assert_true(raw_read.info.get('subject_info') is None)
 
 
-def test_get_chpi():
-    """Test CHPI position computation
-    """
-    trans0, rot0, _ = get_chpi_positions(hp_fname)
-    raw = Raw(hp_fif_fname)
-    out = get_chpi_positions(raw)
-    trans1, rot1, t1 = out
-    trans1 = trans1[2:]
-    rot1 = rot1[2:]
-    # these will not be exact because they don't use equiv. time points
-    assert_allclose(trans0, trans1, atol=1e-6, rtol=1e-1)
-    assert_allclose(rot0, rot1, atol=1e-6, rtol=1e-1)
-    # run through input checking
-    assert_raises(TypeError, get_chpi_positions, 1)
-    assert_raises(ValueError, get_chpi_positions, hp_fname, [1])
-
-
+@testing.requires_testing_data
 def test_copy_append():
     """Test raw copying and appending combinations
     """
@@ -109,47 +138,74 @@ def test_copy_append():
     raw_full = Raw(fif_fname)
     raw_full.append(raw)
     data = raw_full[:, :][0]
-    assert_true(data.shape[1] == 2 * raw._data.shape[1])
+    assert_equal(data.shape[1], 2 * raw._data.shape[1])
 
 
+@slow_test
+@testing.requires_testing_data
 def test_rank_estimation():
     """Test raw rank estimation
     """
-    raw = Raw(fif_fname)
-    picks_meg = pick_types(raw.info, meg=True, eeg=False, exclude='bads')
-    n_meg = len(picks_meg)
-    picks_eeg = pick_types(raw.info, meg=False, eeg=True, exclude='bads')
-    n_eeg = len(picks_eeg)
-    raw = Raw(fif_fname, preload=True)
-    assert_array_equal(raw.estimate_rank(), n_meg + n_eeg)
-    assert_array_equal(raw.estimate_rank(picks=picks_eeg), n_eeg)
-    raw = Raw(fif_fname, preload=False)
-    raw.apply_proj()
-    n_proj = len(raw.info['projs'])
-    assert_array_equal(raw.estimate_rank(tstart=10, tstop=20),
-                       n_meg + n_eeg - n_proj)
+    iter_tests = itt.product(
+        [fif_fname, hp_fif_fname],  # sss
+        ['norm', dict(mag=1e11, grad=1e9, eeg=1e5)]
+    )
+    for fname, scalings in iter_tests:
+        raw = Raw(fname)
+        (_, picks_meg), (_, picks_eeg) = _picks_by_type(raw.info,
+                                                        meg_combined=True)
+        n_meg = len(picks_meg)
+        n_eeg = len(picks_eeg)
+
+        raw = Raw(fname, preload=True)
+        if 'proc_history' not in raw.info:
+            expected_rank = n_meg + n_eeg
+        else:
+            mf = raw.info['proc_history'][0]['max_info']
+            expected_rank = _get_sss_rank(mf) + n_eeg
+        assert_array_equal(raw.estimate_rank(scalings=scalings), expected_rank)
+
+        assert_array_equal(raw.estimate_rank(picks=picks_eeg,
+                                             scalings=scalings),
+                           n_eeg)
+
+        raw = Raw(fname, preload=False)
+        if 'sss' in fname:
+            tstart, tstop = 0., 30.
+            raw.add_proj(compute_proj_raw(raw))
+            raw.apply_proj()
+        else:
+            tstart, tstop = 10., 20.
+
+        raw.apply_proj()
+        n_proj = len(raw.info['projs'])
+
+        assert_array_equal(raw.estimate_rank(tstart=tstart, tstop=tstop,
+                                             scalings=scalings),
+                           expected_rank - (1 if 'sss' in fname else n_proj))
 
 
+@testing.requires_testing_data
 def test_output_formats():
     """Test saving and loading raw data using multiple formats
     """
+    tempdir = _TempDir()
     formats = ['short', 'int', 'single', 'double']
     tols = [1e-4, 1e-7, 1e-7, 1e-15]
 
     # let's fake a raw file with different formats
-    raw = Raw(fif_fname, preload=True)
-    raw.crop(0, 1, copy=False)
+    raw = Raw(test_fif_fname).crop(0, 1, copy=False)
 
     temp_file = op.join(tempdir, 'raw.fif')
-    for ii, (format, tol) in enumerate(zip(formats, tols)):
+    for ii, (fmt, tol) in enumerate(zip(formats, tols)):
         # Let's test the overwriting error throwing while we're at it
         if ii > 0:
-            assert_raises(IOError, raw.save, temp_file, format=format)
-        raw.save(temp_file, format=format, overwrite=True)
+            assert_raises(IOError, raw.save, temp_file, fmt=fmt)
+        raw.save(temp_file, fmt=fmt, overwrite=True)
         raw2 = Raw(temp_file)
         raw2_data = raw2[:, :][0]
-        assert_allclose(raw2_data, raw._data, rtol=tol, atol=1e-25)
-        assert_true(raw2.orig_format == format)
+        assert_allclose(raw2_data, raw[:, :][0], rtol=tol, atol=1e-25)
+        assert_equal(raw2.orig_format, fmt)
 
 
 def _compare_combo(raw, new, times, n_times):
@@ -159,11 +215,16 @@ def _compare_combo(raw, new, times, n_times):
         assert_allclose(orig, new[:, ti][0])
 
 
+@slow_test
+@testing.requires_testing_data
 def test_multiple_files():
     """Test loading multiple files simultaneously
     """
     # split file
-    raw = Raw(fif_fname, preload=True).crop(0, 10)
+    tempdir = _TempDir()
+    raw = Raw(fif_fname).crop(0, 10, False)
+    raw.load_data()
+    raw.load_data()  # test no operation
     split_size = 3.  # in seconds
     sfreq = raw.info['sfreq']
     nsamp = (raw.last_samp - raw.first_samp)
@@ -171,7 +232,7 @@ def test_multiple_files():
     tmaxs = np.concatenate((tmins[1:] - 1, [nsamp]))
     tmaxs /= sfreq
     tmins /= sfreq
-    assert_equal(raw.n_times, len(raw._times))
+    assert_equal(raw.n_times, len(raw.times))
 
     # going in reverse order so the last fname is the first file (need later)
     raws = [None] * len(tmins)
@@ -187,8 +248,8 @@ def test_multiple_files():
     assert_raises(ValueError, concatenate_raws, raws, True, events[1:])
     all_raw_1, events1 = concatenate_raws(raws, preload=False,
                                           events_list=events)
-    assert_true(raw.first_samp == all_raw_1.first_samp)
-    assert_true(raw.last_samp == all_raw_1.last_samp)
+    assert_equal(raw.first_samp, all_raw_1.first_samp)
+    assert_equal(raw.last_samp, all_raw_1.last_samp)
     assert_allclose(raw[:, :][0], all_raw_1[:, :][0])
     raws[0] = Raw(fname)
     all_raw_2 = concatenate_raws(raws, preload=True)
@@ -202,7 +263,7 @@ def test_multiple_files():
 
     # test various methods of combining files
     raw = Raw(fif_fname, preload=True)
-    n_times = len(raw._times)
+    n_times = raw.n_times
     # make sure that all our data match
     times = list(range(0, 2 * n_times, 999))
     # add potentially problematic points
@@ -216,14 +277,14 @@ def test_multiple_files():
     _compare_combo(raw, raw_combo, times, n_times)
     assert_raises(ValueError, Raw, [fif_fname, ctf_fname])
     assert_raises(ValueError, Raw, [fif_fname, fif_bad_marked_fname])
-    assert_true(raw[:, :][0].shape[1] * 2 == raw_combo0[:, :][0].shape[1])
-    assert_true(raw_combo0[:, :][0].shape[1] == len(raw_combo0._times))
+    assert_equal(raw[:, :][0].shape[1] * 2, raw_combo0[:, :][0].shape[1])
+    assert_equal(raw_combo0[:, :][0].shape[1], raw_combo0.n_times)
 
     # with all data preloaded, result should be preloaded
     raw_combo = Raw(fif_fname, preload=True)
     raw_combo.append(Raw(fif_fname, preload=True))
     assert_true(raw_combo.preload is True)
-    assert_true(len(raw_combo._times) == raw_combo._data.shape[1])
+    assert_equal(raw_combo.n_times, raw_combo._data.shape[1])
     _compare_combo(raw, raw_combo, times, n_times)
 
     # with any data not preloaded, don't set result as preloaded
@@ -270,18 +331,22 @@ def test_multiple_files():
     assert_array_equal(events, events2)
 
     # check out the len method
-    assert_true(len(raw) == raw.n_times)
-    assert_true(len(raw) == raw.last_samp - raw.first_samp + 1)
+    assert_equal(len(raw), raw.n_times)
+    assert_equal(len(raw), raw.last_samp - raw.first_samp + 1)
 
 
+@testing.requires_testing_data
 def test_split_files():
     """Test writing and reading of split raw files
     """
+    tempdir = _TempDir()
     raw_1 = Raw(fif_fname, preload=True)
+    assert_allclose(raw_1.info['buffer_size_sec'], 10., atol=1e-2)  # samp rate
     split_fname = op.join(tempdir, 'split_raw.fif')
     raw_1.save(split_fname, buffer_size_sec=1.0, split_size='10MB')
 
     raw_2 = Raw(split_fname)
+    assert_allclose(raw_2.info['buffer_size_sec'], 1., atol=1e-2)  # samp rate
     data_1, times_1 = raw_1[:, :]
     data_2, times_2 = raw_2[:, :]
     assert_array_equal(data_1, data_2)
@@ -301,10 +366,11 @@ def test_split_files():
 def test_load_bad_channels():
     """Test reading/writing of bad channels
     """
+    tempdir = _TempDir()
     # Load correctly marked file (manually done in mne_process_raw)
     raw_marked = Raw(fif_bad_marked_fname)
     correct_bads = raw_marked.info['bads']
-    raw = Raw(fif_fname)
+    raw = Raw(test_fif_fname)
     # Make sure it starts clean
     assert_array_equal(raw.info['bads'], [])
 
@@ -338,12 +404,16 @@ def test_load_bad_channels():
     assert_equal([], raw_new.info['bads'])
 
 
+@slow_test
+@testing.requires_testing_data
 def test_io_raw():
     """Test IO for raw data (Neuromag + CTF + gz)
     """
+    tempdir = _TempDir()
     # test unicode io
     for chars in [b'\xc3\xa4\xc3\xb6\xc3\xa9', b'a']:
         with Raw(fif_fname) as r:
+            assert_true('Raw' in repr(r))
             desc1 = r.info['description'] = chars.decode('utf-8')
             temp_file = op.join(tempdir, 'raw.fif')
             r.save(temp_file, overwrite=True)
@@ -352,8 +422,8 @@ def test_io_raw():
             assert_equal(desc1, desc2)
 
     # Let's construct a simple test for IO first
-    raw = Raw(fif_fname, preload=True)
-    raw.crop(0, 3.5)
+    raw = Raw(fif_fname).crop(0, 3.5, False)
+    raw.load_data()
     # put in some data that we know the values of
     data = np.random.randn(raw._data.shape[0], raw._data.shape[1])
     raw._data[:, :] = data
@@ -362,14 +432,14 @@ def test_io_raw():
     raw.save(fname, buffer_size_sec=1.0)
     # read it in, make sure the whole thing matches
     raw = Raw(fname)
-    assert_true(np.allclose(data, raw[:, :][0], 1e-6, 1e-20))
+    assert_allclose(data, raw[:, :][0], rtol=1e-6, atol=1e-20)
     # let's read portions across the 1-sec tag boundary, too
     inds = raw.time_as_index([1.75, 2.25])
     sl = slice(inds[0], inds[1])
-    assert_true(np.allclose(data[:, sl], raw[:, sl][0], 1e-6, 1e-20))
+    assert_allclose(data[:, sl], raw[:, sl][0], rtol=1e-6, atol=1e-20)
 
     # now let's do some real I/O
-    fnames_in = [fif_fname, fif_gz_fname, ctf_fname]
+    fnames_in = [fif_fname, test_fif_gz_fname, ctf_fname]
     fnames_out = ['raw.fif', 'raw.fif.gz', 'raw.fif']
     for fname_in, fname_out in zip(fnames_in, fnames_out):
         fname_out = op.join(tempdir, fname_out)
@@ -395,7 +465,7 @@ def test_io_raw():
         # Writing with drop_small_buffer True
         raw.save(fname_out, picks, tmin=0, tmax=4, buffer_size_sec=3,
                  drop_small_buffer=True, overwrite=True)
-        raw2 = Raw(fname_out, preload=True)
+        raw2 = Raw(fname_out)
 
         sel = pick_channels(raw2.ch_names, meg_ch_names)
         data2, times2 = raw2[sel, :]
@@ -405,14 +475,14 @@ def test_io_raw():
         raw.save(fname_out, picks, tmin=0, tmax=5, overwrite=True)
 
         if fname_in == fif_fname or fname_in == fif_fname + '.gz':
-            assert_true(len(raw.info['dig']) == 146)
+            assert_equal(len(raw.info['dig']), 146)
 
         raw2 = Raw(fname_out)
 
         sel = pick_channels(raw2.ch_names, meg_ch_names)
         data2, times2 = raw2[sel, :]
 
-        assert_true(np.allclose(data, data2, 1e-6, 1e-20))
+        assert_allclose(data, data2, rtol=1e-6, atol=1e-20)
         assert_allclose(times, times2)
         assert_allclose(raw.info['sfreq'], raw2.info['sfreq'], rtol=1e-5)
 
@@ -434,8 +504,8 @@ def test_io_raw():
                 else:
                     to_id = FIFF.FIFFV_MNE_COORD_CTF_HEAD
                 for raw_ in [raw, raw2]:
-                    assert_true(raw_.info[trans]['from'] == from_id)
-                    assert_true(raw_.info[trans]['to'] == to_id)
+                    assert_equal(raw_.info[trans]['from'], from_id)
+                    assert_equal(raw_.info[trans]['to'], to_id)
 
         if fname_in == fif_fname or fname_in == fif_fname + '.gz':
             assert_allclose(raw.info['dig'][0]['r'], raw2.info['dig'][0]['r'])
@@ -449,9 +519,11 @@ def test_io_raw():
     assert_true(len(w) > 0)  # len(w) should be 2 but Travis sometimes has more
 
 
+@testing.requires_testing_data
 def test_io_complex():
     """Test IO with complex data types
     """
+    tempdir = _TempDir()
     dtypes = [np.complex64, np.complex128]
 
     raw = Raw(fif_fname, preload=True)
@@ -486,6 +558,7 @@ def test_io_complex():
         assert_allclose(raw2_data[:, :n_samp], raw_cp._data[picks, :n_samp])
 
 
+@testing.requires_testing_data
 def test_getitem():
     """Test getitem/indexing of Raw
     """
@@ -504,9 +577,11 @@ def test_getitem():
         assert_array_equal(times, times1)
 
 
+@testing.requires_testing_data
 def test_proj():
     """Test SSP proj operations
     """
+    tempdir = _TempDir()
     for proj in [True, False]:
         raw = Raw(fif_fname, preload=False, proj=proj)
         assert_true(all(p['active'] == proj for p in raw.info['projs']))
@@ -525,11 +600,11 @@ def test_proj():
             projs = deepcopy(raw.info['projs'])
             n_proj = len(raw.info['projs'])
             raw.del_proj(0)
-            assert_true(len(raw.info['projs']) == n_proj - 1)
+            assert_equal(len(raw.info['projs']), n_proj - 1)
             raw.add_proj(projs, remove_existing=False)
-            assert_true(len(raw.info['projs']) == 2 * n_proj - 1)
+            assert_equal(len(raw.info['projs']), 2 * n_proj - 1)
             raw.add_proj(projs, remove_existing=True)
-            assert_true(len(raw.info['projs']) == n_proj)
+            assert_equal(len(raw.info['projs']), n_proj)
 
     # test apply_proj() with and without preload
     for preload in [True, False]:
@@ -560,10 +635,23 @@ def test_proj():
         assert_allclose(data_proj_1, data_proj_2)
         assert_allclose(data_proj_2, np.dot(raw._projector, data_proj_2))
 
+    tempdir = _TempDir()
+    out_fname = op.join(tempdir, 'test_raw.fif')
+    raw = read_raw_fif(test_fif_fname, preload=True).crop(0, 0.002, copy=False)
+    raw.pick_types(meg=False, eeg=True)
+    raw.info['projs'] = [raw.info['projs'][-1]]
+    raw._data.fill(0)
+    raw._data[-1] = 1.
+    raw.save(out_fname)
+    raw = read_raw_fif(out_fname, proj=True, preload=False)
+    assert_allclose(raw[:, :][0][:1], raw[0, :][0])
 
+
+@testing.requires_testing_data
 def test_preload_modify():
     """Test preloading and modifying data
     """
+    tempdir = _TempDir()
     for preload in [False, True, 'memmap.dat']:
         raw = Raw(fif_fname, preload=preload)
 
@@ -589,10 +677,13 @@ def test_preload_modify():
         assert_allclose(data, data_new)
 
 
+@slow_test
+@testing.requires_testing_data
 def test_filter():
     """Test filtering (FIR and IIR) and Raw.apply_function interface
     """
-    raw = Raw(fif_fname, preload=True).crop(0, 7, False)
+    raw = Raw(fif_fname).crop(0, 7, False)
+    raw.load_data()
     sig_dec = 11
     sig_dec_notch = 12
     sig_dec_notch_fit = 12
@@ -660,6 +751,7 @@ def test_filter():
     assert_array_almost_equal(data, data_notch, sig_dec_notch_fit)
 
 
+@testing.requires_testing_data
 def test_crop():
     """Test cropping raw files
     """
@@ -679,8 +771,8 @@ def test_crop():
     for ri, (tmin, tmax) in enumerate(zip(tmins, tmaxs)):
         raws[ri] = raw.crop(tmin, tmax, True)
     all_raw_2 = concatenate_raws(raws, preload=False)
-    assert_true(raw.first_samp == all_raw_2.first_samp)
-    assert_true(raw.last_samp == all_raw_2.last_samp)
+    assert_equal(raw.first_samp, all_raw_2.first_samp)
+    assert_equal(raw.last_samp, all_raw_2.last_samp)
     assert_array_equal(raw[:, :][0], all_raw_2[:, :][0])
 
     tmins = np.round(np.arange(0., nsamp - 1, split_size * sfreq))
@@ -698,32 +790,35 @@ def test_crop():
 
     all_raw_2 = raw.crop(0, None, True)
     for ar in [all_raw_1, all_raw_2]:
-        assert_true(raw.first_samp == ar.first_samp)
-        assert_true(raw.last_samp == ar.last_samp)
+        assert_equal(raw.first_samp, ar.first_samp)
+        assert_equal(raw.last_samp, ar.last_samp)
         assert_array_equal(raw[:, :][0], ar[:, :][0])
 
 
+@testing.requires_testing_data
 def test_resample():
     """Test resample (with I/O and multiple files)
     """
-    raw = Raw(fif_fname, preload=True).crop(0, 3, False)
+    tempdir = _TempDir()
+    raw = Raw(fif_fname).crop(0, 3, False)
+    raw.load_data()
     raw_resamp = raw.copy()
     sfreq = raw.info['sfreq']
     # test parallel on upsample
     raw_resamp.resample(sfreq * 2, n_jobs=2)
-    assert_true(raw_resamp.n_times == len(raw_resamp._times))
+    assert_equal(raw_resamp.n_times, len(raw_resamp.times))
     raw_resamp.save(op.join(tempdir, 'raw_resamp-raw.fif'))
     raw_resamp = Raw(op.join(tempdir, 'raw_resamp-raw.fif'), preload=True)
-    assert_true(sfreq == raw_resamp.info['sfreq'] / 2)
-    assert_true(raw.n_times == raw_resamp.n_times / 2)
-    assert_true(raw_resamp._data.shape[1] == raw_resamp.n_times)
-    assert_true(raw._data.shape[0] == raw_resamp._data.shape[0])
+    assert_equal(sfreq, raw_resamp.info['sfreq'] / 2)
+    assert_equal(raw.n_times, raw_resamp.n_times / 2)
+    assert_equal(raw_resamp._data.shape[1], raw_resamp.n_times)
+    assert_equal(raw._data.shape[0], raw_resamp._data.shape[0])
     # test non-parallel on downsample
     raw_resamp.resample(sfreq, n_jobs=1)
-    assert_true(raw_resamp.info['sfreq'] == sfreq)
-    assert_true(raw._data.shape == raw_resamp._data.shape)
-    assert_true(raw.first_samp == raw_resamp.first_samp)
-    assert_true(raw.last_samp == raw.last_samp)
+    assert_equal(raw_resamp.info['sfreq'], sfreq)
+    assert_equal(raw._data.shape, raw_resamp._data.shape)
+    assert_equal(raw.first_samp, raw_resamp.first_samp)
+    assert_equal(raw.last_samp, raw.last_samp)
     # upsampling then downsampling doubles resampling error, but this still
     # works (hooray). Note that the stim channels had to be sub-sampled
     # without filtering to be accurately preserved
@@ -742,9 +837,9 @@ def test_resample():
     raw3 = raw.copy()
     raw4 = raw.copy()
     raw1 = concatenate_raws([raw1, raw2])
-    raw1.resample(10)
-    raw3.resample(10)
-    raw4.resample(10)
+    raw1.resample(10.)
+    raw3.resample(10.)
+    raw4.resample(10.)
     raw3 = concatenate_raws([raw3, raw4])
     assert_array_equal(raw1._data, raw3._data)
     assert_array_equal(raw1._first_samps, raw3._first_samps)
@@ -754,7 +849,65 @@ def test_resample():
     assert_equal(raw1.last_samp, raw3.last_samp)
     assert_equal(raw1.info['sfreq'], raw3.info['sfreq'])
 
+    # test resampling of stim channel
 
+    # basic decimation
+    stim = [1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    assert_allclose(raw.resample(8.)._data,
+                    [[1, 1, 0, 0, 1, 1, 0, 0]])
+
+    # decimation of multiple stim channels
+    raw = RawArray(2 * [stim], create_info(2, len(stim), 2 * ['stim']))
+    assert_allclose(raw.resample(8.)._data,
+                    [[1, 1, 0, 0, 1, 1, 0, 0],
+                     [1, 1, 0, 0, 1, 1, 0, 0]])
+
+    # decimation that could potentially drop events if the decimation is
+    # done naively
+    stim = [0, 0, 0, 1, 1, 0, 0, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    assert_allclose(raw.resample(4.)._data,
+                    [[0, 1, 1, 0]])
+
+    # two events are merged in this case (warning)
+    stim = [0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        raw.resample(8.)
+        assert_true(len(w) == 1)
+
+    # events are dropped in this case (warning)
+    stim = [0, 1, 1, 0, 0, 1, 1, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        raw.resample(4.)
+        assert_true(len(w) == 1)
+
+    # test resampling events: this should no longer give a warning
+    stim = [0, 1, 1, 0, 0, 1, 1, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    events = find_events(raw)
+    raw, events = raw.resample(4., events=events)
+    assert_equal(events, np.array([[0, 0, 1], [2, 0, 1]]))
+
+    # test copy flag
+    stim = [1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0]
+    raw = RawArray([stim], create_info(1, len(stim), ['stim']))
+    raw_resampled = raw.resample(4., copy=True)
+    assert_true(raw_resampled is not raw)
+    raw_resampled = raw.resample(4., copy=False)
+    assert_true(raw_resampled is raw)
+
+    # resample should still work even when no stim channel is present
+    raw = RawArray(np.random.randn(1, 100), create_info(1, 100, ['eeg']))
+    raw.resample(10)
+    assert_true(len(raw) == 10)
+
+
+@testing.requires_testing_data
 def test_hilbert():
     """Test computation of analytic signal using hilbert
     """
@@ -762,14 +915,29 @@ def test_hilbert():
     picks_meg = pick_types(raw.info, meg=True, exclude='bads')
     picks = picks_meg[:4]
 
+    raw_filt = raw.copy()
+    raw_filt.filter(10, 20)
+    raw_filt_2 = raw_filt.copy()
+
     raw2 = raw.copy()
+    raw3 = raw.copy()
     raw.apply_hilbert(picks)
     raw2.apply_hilbert(picks, envelope=True, n_jobs=2)
 
+    # Test custom n_fft
+    raw_filt.apply_hilbert(picks)
+    raw_filt_2.apply_hilbert(picks, n_fft=raw_filt_2.n_times + 1000)
+    assert_equal(raw_filt._data.shape, raw_filt_2._data.shape)
+    assert_allclose(raw_filt._data[:, 50:-50], raw_filt_2._data[:, 50:-50],
+                    atol=1e-13, rtol=1e-2)
+    assert_raises(ValueError, raw3.apply_hilbert, picks,
+                  n_fft=raw3.n_times - 100)
+
     env = np.abs(raw._data[picks, :])
     assert_allclose(env, raw2._data[picks, :], rtol=1e-2, atol=1e-13)
 
 
+@testing.requires_testing_data
 def test_raw_copy():
     """Test Raw copy
     """
@@ -790,53 +958,28 @@ def test_raw_copy():
                  sorted(copied.__dict__.keys()))
 
 
-@requires_nitime
-def test_raw_to_nitime():
-    """ Test nitime export """
-    raw = Raw(fif_fname, preload=True)
-    picks_meg = pick_types(raw.info, meg=True, exclude='bads')
-    picks = picks_meg[:4]
-    raw_ts = raw.to_nitime(picks=picks)
-    assert_true(raw_ts.data.shape[0] == len(picks))
-
-    raw = Raw(fif_fname, preload=False)
-    picks_meg = pick_types(raw.info, meg=True, exclude='bads')
-    picks = picks_meg[:4]
-    raw_ts = raw.to_nitime(picks=picks)
-    assert_true(raw_ts.data.shape[0] == len(picks))
-
-    raw = Raw(fif_fname, preload=True)
-    picks_meg = pick_types(raw.info, meg=True, exclude='bads')
-    picks = picks_meg[:4]
-    raw_ts = raw.to_nitime(picks=picks, copy=False)
-    assert_true(raw_ts.data.shape[0] == len(picks))
-
-    raw = Raw(fif_fname, preload=False)
-    picks_meg = pick_types(raw.info, meg=True, exclude='bads')
-    picks = picks_meg[:4]
-    raw_ts = raw.to_nitime(picks=picks, copy=False)
-    assert_true(raw_ts.data.shape[0] == len(picks))
-
-
 @requires_pandas
-def test_as_data_frame():
+def test_to_data_frame():
     """Test raw Pandas exporter"""
-    raw = Raw(fif_fname, preload=True)
-    df = raw.as_data_frame()
+    raw = Raw(test_fif_fname, preload=True)
+    _, times = raw[0, :10]
+    df = raw.to_data_frame()
     assert_true((df.columns == raw.ch_names).all())
-    df = raw.as_data_frame(use_time_index=False)
-    assert_true('time' in df.columns)
-    assert_array_equal(df.values[:, 1], raw._data[0] * 1e13)
-    assert_array_equal(df.values[:, 3], raw._data[2] * 1e15)
+    assert_array_equal(np.round(times * 1e3), df.index.values[:10])
+    df = raw.to_data_frame(index=None)
+    assert_true('time' in df.index.names)
+    assert_array_equal(df.values[:, 0], raw._data[0] * 1e13)
+    assert_array_equal(df.values[:, 2], raw._data[2] * 1e15)
 
 
+@testing.requires_testing_data
 def test_raw_index_as_time():
     """ Test index as time conversion"""
     raw = Raw(fif_fname, preload=True)
     t0 = raw.index_as_time([0], True)[0]
     t1 = raw.index_as_time([100], False)[0]
     t2 = raw.index_as_time([100], True)[0]
-    assert_true((t2 - t1) == t0)
+    assert_equal(t2 - t1, t0)
     # ensure we can go back and forth
     t3 = raw.index_as_time(raw.time_as_index([0], True), True)
     assert_array_almost_equal(t3, [0.0], 2)
@@ -845,23 +988,57 @@ def test_raw_index_as_time():
     t3 = raw.index_as_time(raw.time_as_index(raw.info['sfreq'], False), False)
     assert_array_almost_equal(t3, [raw.info['sfreq']], 2)
     i0 = raw.time_as_index(raw.index_as_time([0], True), True)
-    assert_true(i0[0] == 0)
+    assert_equal(i0[0], 0)
     i1 = raw.time_as_index(raw.index_as_time([100], True), True)
-    assert_true(i1[0] == 100)
+    assert_equal(i1[0], 100)
     # Have to add small amount of time because we truncate via int casting
     i1 = raw.time_as_index(raw.index_as_time([100.0001], False), False)
-    assert_true(i1[0] == 100)
+    assert_equal(i1[0], 100)
 
 
+def test_add_channels():
+    """Test raw splitting / re-appending channel types
+    """
+    raw = Raw(test_fif_fname).crop(0, 1).load_data()
+    raw_nopre = Raw(test_fif_fname, preload=False)
+    raw_eeg_meg = raw.pick_types(meg=True, eeg=True, copy=True)
+    raw_eeg = raw.pick_types(meg=False, eeg=True, copy=True)
+    raw_meg = raw.pick_types(meg=True, eeg=False, copy=True)
+    raw_stim = raw.pick_types(meg=False, eeg=False, stim=True, copy=True)
+    raw_new = raw_meg.add_channels([raw_eeg, raw_stim], copy=True)
+    assert_true(all(ch in raw_new.ch_names
+                    for ch in raw_stim.ch_names + raw_meg.ch_names))
+    raw_new = raw_meg.add_channels([raw_eeg], copy=True)
+
+    assert_true(ch in raw_new.ch_names for ch in raw.ch_names)
+    assert_array_equal(raw_new[:, :][0], raw_eeg_meg[:, :][0])
+    assert_array_equal(raw_new[:, :][1], raw[:, :][1])
+    assert_true(all(ch not in raw_new.ch_names for ch in raw_stim.ch_names))
+
+    # Now test errors
+    raw_badsf = raw_eeg.copy()
+    raw_badsf.info['sfreq'] = 3.1415927
+    raw_eeg = raw_eeg.crop(.5)
+
+    assert_raises(AssertionError, raw_meg.add_channels, [raw_nopre])
+    assert_raises(RuntimeError, raw_meg.add_channels, [raw_badsf])
+    assert_raises(AssertionError, raw_meg.add_channels, [raw_eeg])
+    assert_raises(ValueError, raw_meg.add_channels, [raw_meg])
+    assert_raises(AssertionError, raw_meg.add_channels, raw_badsf)
+
+
+@testing.requires_testing_data
 def test_raw_time_as_index():
     """ Test time as index conversion"""
     raw = Raw(fif_fname, preload=True)
     first_samp = raw.time_as_index([0], True)[0]
-    assert_true(raw.first_samp == -first_samp)
+    assert_equal(raw.first_samp, -first_samp)
 
 
+@testing.requires_testing_data
 def test_save():
     """ Test saving raw"""
+    tempdir = _TempDir()
     raw = Raw(fif_fname, preload=False)
     # can't write over file being read
     assert_raises(ValueError, raw.save, fif_fname)
@@ -880,6 +1057,7 @@ def test_save():
     os.remove(new_fname)
 
 
+@testing.requires_testing_data
 def test_with_statement():
     """ Test with statement """
     for preload in [True, False]:
@@ -890,6 +1068,7 @@ def test_with_statement():
 def test_compensation_raw():
     """Test Raw compensation
     """
+    tempdir = _TempDir()
     raw1 = Raw(ctf_comp_fname, compensation=None)
     assert_true(raw1.comp is None)
     data1, times1 = raw1[:, :]
@@ -929,6 +1108,8 @@ def test_compensation_raw():
 def test_compensation_raw_mne():
     """Test Raw compensation by comparing with MNE
     """
+    tempdir = _TempDir()
+
     def compensate_mne(fname, grad):
         tmp_fname = op.join(tempdir, 'mne_ctf_test_raw.fif')
         cmd = ['mne_process_raw', '--raw', fname, '--save', tmp_fname,
@@ -942,38 +1123,7 @@ def test_compensation_raw_mne():
         assert_allclose(raw_py._data, raw_c._data, rtol=1e-6, atol=1e-17)
 
 
-def test_set_eeg_reference():
-    """ Test rereference eeg data"""
-    raw = Raw(fif_fname, preload=True)
-
-    # Rereference raw data by creating a copy of original data
-    reref, ref_data = set_eeg_reference(raw, ['EEG 001', 'EEG 002'], copy=True)
-
-    # Separate EEG channels from other channel types
-    picks_eeg = pick_types(raw.info, meg=False, eeg=True, exclude='bads')
-    picks_other = pick_types(raw.info, meg=True, eeg=False, eog=True,
-                             stim=True, exclude='bads')
-
-    # Get the raw EEG data and other channel data
-    raw_eeg_data = raw[picks_eeg][0]
-    raw_other_data = raw[picks_other][0]
-
-    # Get the rereferenced EEG data and channel other
-    reref_eeg_data = reref[picks_eeg][0]
-    unref_eeg_data = reref_eeg_data + ref_data
-    # Undo rereferencing of EEG channels
-    reref_other_data = reref[picks_other][0]
-
-    # Check that both EEG data and other data is the same
-    assert_array_equal(raw_eeg_data, unref_eeg_data)
-    assert_array_equal(raw_other_data, reref_other_data)
-
-    # Test that data is modified in place when copy=False
-    reref, ref_data = set_eeg_reference(raw, ['EEG 001', 'EEG 002'],
-                                        copy=False)
-    assert_true(raw is reref)
-
-
+@testing.requires_testing_data
 def test_drop_channels_mixin():
     """Test channels-dropping functionality
     """
@@ -989,10 +1139,11 @@ def test_drop_channels_mixin():
 
     raw.drop_channels(drop_ch)
     assert_equal(ch_names, raw.ch_names)
-    assert_equal(len(ch_names), len(raw.cals))
+    assert_equal(len(ch_names), len(raw._cals))
     assert_equal(len(ch_names), raw._data.shape[0])
 
 
+@testing.requires_testing_data
 def test_pick_channels_mixin():
     """Test channel-picking functionality
     """
@@ -1009,14 +1160,16 @@ def test_pick_channels_mixin():
 
     raw.pick_channels(ch_names, copy=False)  # copy is False
     assert_equal(ch_names, raw.ch_names)
-    assert_equal(len(ch_names), len(raw.cals))
+    assert_equal(len(ch_names), len(raw._cals))
     assert_equal(len(ch_names), raw._data.shape[0])
+    assert_raises(ValueError, raw.pick_channels, ch_names[0])
 
     raw = Raw(fif_fname, preload=False)
     assert_raises(RuntimeError, raw.pick_channels, ch_names)
     assert_raises(RuntimeError, raw.drop_channels, ch_names)
 
 
+@testing.requires_testing_data
 def test_equalize_channels():
     """Test equalization of channels
     """
@@ -1030,3 +1183,6 @@ def test_equalize_channels():
     equalize_channels(my_comparison)
     for e in my_comparison:
         assert_equal(ch_names, e.ch_names)
+
+
+run_tests_if_main()
