@@ -27,7 +27,8 @@ def channel_type(info, idx):
     Returns
     -------
     type : 'grad' | 'mag' | 'eeg' | 'stim' | 'eog' | 'emg' | 'ecg'
-           'ref_meg' | 'resp' | 'exci' | 'ias' | 'syst'
+           'ref_meg' | 'resp' | 'exci' | 'ias' | 'syst' | 'misc'
+           'seeg' | 'chpi'
         Type of channel
     """
     kind = info['chs'][idx]['kind']
@@ -58,6 +59,8 @@ def channel_type(info, idx):
         return 'ias'
     elif kind == FIFF.FIFFV_SYST_CH:
         return 'syst'
+    elif kind == FIFF.FIFFV_SEEG_CH:
+        return 'seeg'
     elif kind in [FIFF.FIFFV_QUAT_0, FIFF.FIFFV_QUAT_1, FIFF.FIFFV_QUAT_2,
                   FIFF.FIFFV_QUAT_3, FIFF.FIFFV_QUAT_4, FIFF.FIFFV_QUAT_5,
                   FIFF.FIFFV_QUAT_6, FIFF.FIFFV_HPI_G, FIFF.FIFFV_HPI_ERR,
@@ -79,6 +82,11 @@ def pick_channels(ch_names, include, exclude=[]):
         List of channels to include (if empty include all available).
     exclude : list of string
         List of channels to exclude (if empty do not exclude any channel).
+        Defaults to [].
+
+    See Also
+    --------
+    pick_channels_regexp, pick_types
 
     Returns
     -------
@@ -87,6 +95,8 @@ def pick_channels(ch_names, include, exclude=[]):
     """
     if len(np.unique(ch_names)) != len(ch_names):
         raise RuntimeError('ch_names is not a unique list, picking is unsafe')
+    _check_excludes_includes(include)
+    _check_excludes_includes(exclude)
     sel = []
     for k, name in enumerate(ch_names):
         if (len(include) == 0 or name in include) and name not in exclude:
@@ -115,6 +125,10 @@ def pick_channels_regexp(ch_names, regexp):
     sel : array of int
         Indices of good channels.
 
+    See Also
+    --------
+    pick_channels
+
     Examples
     --------
     >>> pick_channels_regexp(['MEG 2331', 'MEG 2332', 'MEG 2333'], 'MEG ...1')
@@ -128,7 +142,7 @@ def pick_channels_regexp(ch_names, regexp):
 
 def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
                emg=False, ref_meg='auto', misc=False, resp=False, chpi=False,
-               exci=False, ias=False, syst=False,
+               exci=False, ias=False, syst=False, seeg=False,
                include=[], exclude='bads', selection=None):
     """Pick channels by type and names
 
@@ -143,14 +157,14 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         gradiometer.
     eeg : bool
         If True include EEG channels.
+    stim : bool
+        If True include stimulus channels.
     eog : bool
         If True include EOG channels.
     ecg : bool
         If True include ECG channels.
     emg : bool
         If True include EMG channels.
-    stim : bool
-        If True include stimulus channels.
     ref_meg: bool | str
         If True include CTF / 4D reference channels. If 'auto', the reference
         channels are only included if compensations are present.
@@ -167,11 +181,13 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         Internal Active Shielding data (maybe on Triux only).
     syst : bool
         System status channel information (on Triux systems only).
+    seeg : bool
+        Stereotactic EEG channels
     include : list of string
         List of additional channels to include. If empty do not include any.
     exclude : list of string | str
-        List of channels to exclude. If empty do not exclude any (default).
-        If 'bads', exclude channels in info['bads'].
+        List of channels to exclude. If 'bads' (default), exclude channels
+        in info['bads'].
     selection : list of string
         Restrict sensor channels (MEG, EEG) to this list of channel names.
 
@@ -180,6 +196,13 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
     sel : array of int
         Indices of good channels.
     """
+    # NOTE: Changes to this function's signature should also be changed in
+    # PickChannelsMixin
+    from .meas_info import Info
+    if not isinstance(info, Info):
+        raise TypeError('info must be an instance of Info, not %s'
+                        % type(info))
+    info._check_consistency()
     nchan = info['nchan']
     pick = np.zeros(nchan, dtype=np.bool)
 
@@ -187,7 +210,7 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         raise ValueError('exclude must be a list of strings or "bads"')
     elif exclude == 'bads':
         exclude = info.get('bads', [])
-    elif not isinstance(exclude, list):
+    elif not isinstance(exclude, (list, tuple)):
         raise ValueError('exclude must either be "bads" or a list of strings.'
                          ' If only one channel is to be excluded, use '
                          '[ch_name] instead of passing ch_name.')
@@ -211,8 +234,8 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
                     pick[k] = True
                 elif meg == 'planar2' and info['ch_names'][k].endswith('3'):
                     pick[k] = True
-            elif (meg == 'mag'
-                    and info['chs'][k]['unit'] == FIFF.FIFF_UNIT_T):
+            elif (meg == 'mag' and
+                  info['chs'][k]['unit'] == FIFF.FIFF_UNIT_T):
                 pick[k] = True
         elif kind == FIFF.FIFFV_EEG_CH and eeg:
             pick[k] = True
@@ -232,6 +255,8 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
             pick[k] = True
         elif kind == FIFF.FIFFV_SYST_CH and syst:
             pick[k] = True
+        elif kind == FIFF.FIFFV_SEEG_CH and seeg:
+            pick[k] = True
         elif kind == FIFF.FIFFV_IAS_CH and ias:
             pick[k] = True
         elif kind == FIFF.FIFFV_EXCI_CH and exci:
@@ -247,9 +272,9 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         # the selection only restricts these types of channels
         sel_kind = [FIFF.FIFFV_MEG_CH, FIFF.FIFFV_REF_MEG_CH,
                     FIFF.FIFFV_EEG_CH]
-        for k in np.where(pick == True)[0]:
-            if (info['chs'][k]['kind'] in sel_kind
-                and info['ch_names'][k] not in selection):
+        for k in np.where(pick == True)[0]:  # noqa
+            if (info['chs'][k]['kind'] in sel_kind and
+                    info['ch_names'][k] not in selection):
                 pick[k] = False
 
     myinclude = [info['ch_names'][k] for k in range(nchan) if pick[k]]
@@ -270,7 +295,7 @@ def pick_info(info, sel=[], copy=True):
     ----------
     info : dict
         Info structure from evoked or raw data.
-    sel : list of int
+    sel : list of int | None
         Indices of channels to include.
     copy : bool
         If copy is False, info is modified inplace.
@@ -280,15 +305,31 @@ def pick_info(info, sel=[], copy=True):
     res : dict
         Info structure restricted to a selection of channels.
     """
+    info._check_consistency()
     if copy:
         info = deepcopy(info)
-
-    if len(sel) == 0:
-        raise ValueError('Warning : No channels match the selection.')
+    if sel is None:
+        return info
+    elif len(sel) == 0:
+        raise ValueError('No channels match the selection.')
 
     info['chs'] = [info['chs'][k] for k in sel]
     info['ch_names'] = [info['ch_names'][k] for k in sel]
     info['nchan'] = len(sel)
+    info['bads'] = [ch for ch in info['bads'] if ch in info['ch_names']]
+
+    comps = deepcopy(info['comps'])
+    for c in comps:
+        row_idx = [k for k, n in enumerate(c['data']['row_names'])
+                   if n in info['ch_names']]
+        row_names = [c['data']['row_names'][i] for i in row_idx]
+        rowcals = c['rowcals'][row_idx]
+        c['rowcals'] = rowcals
+        c['data']['nrow'] = len(row_names)
+        c['data']['row_names'] = row_names
+        c['data']['data'] = c['data']['data'][row_idx]
+    info['comps'] = comps
+
     return info
 
 
@@ -313,9 +354,9 @@ def pick_channels_evoked(orig, include=[], exclude='bads'):
         One evoked dataset.
     include : list of string, (optional)
         List of channels to include (if empty, include all available).
-    exclude : list of string, (optional) | 'bads'
-        Channels to exclude (if empty, do not exclude any).
-         Defaults to 'bads'.
+    exclude : list of string | str
+        List of channels to exclude. If empty do not exclude any (default).
+        If 'bads', exclude channels in orig.info['bads']. Defaults to 'bads'.
 
     Returns
     -------
@@ -323,10 +364,11 @@ def pick_channels_evoked(orig, include=[], exclude='bads'):
         Evoked data restricted to selected channels. If include and
         exclude are empty it returns orig without copy.
     """
-
     if len(include) == 0 and len(exclude) == 0:
         return orig
 
+    exclude = _check_excludes_includes(exclude, info=orig.info,
+                                       allow_bads=True)
     sel = pick_channels(orig.info['ch_names'], include=include,
                         exclude=exclude)
 
@@ -346,66 +388,6 @@ def pick_channels_evoked(orig, include=[], exclude='bads'):
     return res
 
 
-def pick_types_evoked(orig, meg=True, eeg=False, stim=False, eog=False,
-                      ecg=False, emg=False, ref_meg=False, misc=False,
-                      resp=False, chpi=False, exci=False, ias=False,
-                      syst=False, include=[], exclude='bads'):
-    """Pick by channel type and names from evoked data
-
-    Parameters
-    ----------
-    info : dict
-        The measurement info
-    meg : bool or string
-        If True include all MEG channels. If False include None
-        If string it can be 'mag' or 'grad' to select only gradiometers
-        or magnetometers.
-    eeg : bool
-        If True include EEG channels
-    eog : bool
-        If True include EOG channels
-    ecg : bool
-        If True include ECG channels
-    emg : bool
-        If True include EMG channels
-    stim : bool
-        If True include stimulus channels
-    ref_meg : bool
-        If True include CTF / 4D reference channels
-    misc : bool
-        If True include miscellaneous analog channels
-    resp : bool
-        If True include response-trigger channel. For some MEG systems this
-        is separate from the stim channel.
-    chpi : bool
-        If True include continuous HPI coil channels.
-    exci : bool
-        Flux excitation channel used to be a stimulus channel.
-    ias : bool
-        Internal Active Shielding data (maybe on Triux only).
-    syst : bool
-        System status channel information (on Triux systems only).
-    include : list of string
-        List of additional channels to include. If empty do not include any.
-    exclude : list of string | str
-        List of channels to exclude. If empty do not exclude any (default).
-        If 'bads', exclude channels in info['bads'].
-
-    Returns
-    -------
-    res : instance of Evoked
-        Evoked data restricted to selected channels. If include and
-        exclude are None it returns orig without copy.
-    """
-    sel = pick_types(info=orig.info, meg=meg, eeg=eeg, stim=stim, eog=eog,
-                     ecg=ecg, emg=emg, ref_meg=ref_meg, misc=misc,
-                     resp=resp, chpi=chpi, exci=exci, ias=ias, syst=syst,
-                     include=include, exclude=exclude)
-
-    include_ch_names = [orig.ch_names[k] for k in sel]
-    return pick_channels_evoked(orig, include_ch_names)
-
-
 @verbose
 def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
     """Pick channels from forward operator
@@ -414,11 +396,12 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
     ----------
     orig : dict
         A forward solution.
-    include : list of string (optional) | None
-        List of channels to include (if empty, include all available). Defaults
-        auto None.
-    exclude : list of string (optional) | None
-        Channels to exclude (if empty, do not exclude any). Defaults to None.
+    include : list of string
+        List of channels to include (if empty, include all available).
+        Defaults to [].
+    exclude : list of string | 'bads'
+        Channels to exclude (if empty, do not exclude any). Defaults to [].
+        If 'bads', then exclude bad channels in orig.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
@@ -428,49 +411,61 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
         Forward solution restricted to selected channels. If include and
         exclude are empty it returns orig without copy.
     """
-
+    orig['info']._check_consistency()
     if len(include) == 0 and len(exclude) == 0:
         return orig
+    exclude = _check_excludes_includes(exclude,
+                                       info=orig['info'], allow_bads=True)
 
-    sel = pick_channels(orig['sol']['row_names'], include=include,
-                        exclude=exclude)
+    # Allow for possibility of channel ordering in forward solution being
+    # different from that of the M/EEG file it is based on.
+    sel_sol = pick_channels(orig['sol']['row_names'], include=include,
+                            exclude=exclude)
+    sel_info = pick_channels(orig['info']['ch_names'], include=include,
+                             exclude=exclude)
 
     fwd = deepcopy(orig)
 
+    # Check that forward solution and original data file agree on #channels
+    if len(sel_sol) != len(sel_info):
+        raise ValueError('Forward solution and functional data appear to '
+                         'have different channel names, please check.')
+
     #   Do we have something?
-    nuse = len(sel)
+    nuse = len(sel_sol)
     if nuse == 0:
         raise ValueError('Nothing remains after picking')
 
     logger.info('    %d out of %d channels remain after picking'
                 % (nuse, fwd['nchan']))
 
-    #   Pick the correct rows of the forward operator
-    fwd['sol']['data'] = fwd['sol']['data'][sel, :]
-    fwd['_orig_sol'] = fwd['_orig_sol'][sel, :]
+    #   Pick the correct rows of the forward operator using sel_sol
+    fwd['sol']['data'] = fwd['sol']['data'][sel_sol, :]
+    fwd['_orig_sol'] = fwd['_orig_sol'][sel_sol, :]
     fwd['sol']['nrow'] = nuse
 
-    ch_names = [fwd['sol']['row_names'][k] for k in sel]
+    ch_names = [fwd['sol']['row_names'][k] for k in sel_sol]
     fwd['nchan'] = nuse
     fwd['sol']['row_names'] = ch_names
 
-    fwd['info']['ch_names'] = [fwd['info']['ch_names'][k] for k in sel]
-    fwd['info']['chs'] = [fwd['info']['chs'][k] for k in sel]
+    # Pick the appropriate channel names from the info-dict using sel_info
+    fwd['info']['ch_names'] = [fwd['info']['ch_names'][k] for k in sel_info]
+    fwd['info']['chs'] = [fwd['info']['chs'][k] for k in sel_info]
     fwd['info']['nchan'] = nuse
     fwd['info']['bads'] = [b for b in fwd['info']['bads'] if b in ch_names]
 
     if fwd['sol_grad'] is not None:
-        fwd['sol_grad']['data'] = fwd['sol_grad']['data'][sel, :]
-        fwd['_orig_sol_grad'] = fwd['_orig_sol_grad'][sel, :]
+        fwd['sol_grad']['data'] = fwd['sol_grad']['data'][sel_sol, :]
+        fwd['_orig_sol_grad'] = fwd['_orig_sol_grad'][sel_sol, :]
         fwd['sol_grad']['nrow'] = nuse
         fwd['sol_grad']['row_names'] = [fwd['sol_grad']['row_names'][k]
-                                        for k in sel]
+                                        for k in sel_sol]
 
     return fwd
 
 
-def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, include=[],
-                       exclude=[]):
+def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, seeg=False,
+                       include=[], exclude=[]):
     """Pick by channel type and names from a forward operator
 
     Parameters
@@ -485,6 +480,8 @@ def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, include=[],
         If True include EEG channels
     ref_meg : bool
         If True include CTF / 4D reference channels
+    seeg : bool
+        If True include stereotactic EEG channels
     include : list of string
         List of additional channels to include. If empty do not include any.
     exclude : list of string | str
@@ -497,8 +494,8 @@ def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, include=[],
         Forward solution restricted to selected channel types.
     """
     info = orig['info']
-    sel = pick_types(info, meg, eeg, ref_meg=ref_meg, include=include,
-                     exclude=exclude)
+    sel = pick_types(info, meg, eeg, ref_meg=ref_meg, seeg=seeg,
+                     include=include, exclude=exclude)
     if len(sel) == 0:
         raise ValueError('No valid channels found')
     include_ch_names = [info['ch_names'][k] for k in sel]
@@ -508,7 +505,9 @@ def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, include=[],
 def channel_indices_by_type(info):
     """Get indices of channels by type
     """
-    idx = dict(grad=[], mag=[], eeg=[], eog=[], ecg=[])
+    idx = dict(grad=[], mag=[], eeg=[], seeg=[], eog=[], ecg=[], stim=[],
+               emg=[], ref_meg=[], misc=[], resp=[], chpi=[], exci=[], ias=[],
+               syst=[])
     for k, ch in enumerate(info['chs']):
         for key in idx.keys():
             if channel_type(info, k) == key:
@@ -534,12 +533,91 @@ def pick_channels_cov(orig, include=[], exclude='bads'):
     res : dict
         Covariance solution restricted to selected channels.
     """
+    exclude = orig['bads'] if exclude == 'bads' else exclude
     sel = pick_channels(orig['names'], include=include, exclude=exclude)
     res = deepcopy(orig)
     res['dim'] = len(sel)
-    res['data'] = orig['data'][sel][:, sel]
+    if not res['diag']:
+        res['data'] = orig['data'][sel][:, sel]
+    else:
+        res['data'] = orig['data'][sel]
     res['names'] = [orig['names'][k] for k in sel]
     res['bads'] = [name for name in orig['bads'] if name in res['names']]
     res['eig'] = None
     res['eigvec'] = None
     return res
+
+
+def _picks_by_type(info, meg_combined=False, ref_meg=False):
+    """Get data channel indices as separate list of tuples
+    Parameters
+    ----------
+    info : instance of mne.measuerment_info.Info
+        The info.
+    meg_combined : bool
+        Whether to return combined picks for grad and mag.
+    ref_meg : bool
+        If True include CTF / 4D reference channels
+
+    Returns
+    -------
+    picks_list : list of tuples
+        The list of tuples of picks and the type string.
+    """
+    from ..channels.channels import _contains_ch_type
+    picks_list = []
+    has_mag, has_grad, has_eeg = [_contains_ch_type(info, k)
+                                  for k in ('mag', 'grad', 'eeg')]
+    if has_mag and (meg_combined is not True or not has_grad):
+        picks_list.append(
+            ('mag', pick_types(info, meg='mag', eeg=False, stim=False,
+             ref_meg=ref_meg))
+        )
+    if has_grad and (meg_combined is not True or not has_mag):
+        picks_list.append(
+            ('grad', pick_types(info, meg='grad', eeg=False, stim=False,
+             ref_meg=ref_meg))
+        )
+    if has_mag and has_grad and meg_combined is True:
+        picks_list.append(
+            ('meg', pick_types(info, meg=True, eeg=False, stim=False,
+             ref_meg=ref_meg))
+        )
+    if has_eeg:
+        picks_list.append(
+            ('eeg', pick_types(info, meg=False, eeg=True, stim=False,
+             ref_meg=ref_meg))
+        )
+    return picks_list
+
+
+def _check_excludes_includes(chs, info=None, allow_bads=False):
+    """Ensure that inputs to exclude/include are list-like or "bads".
+
+    Parameters
+    ----------
+    chs : any input, should be list, tuple, string
+        The channels passed to include or exclude.
+    allow_bads : bool
+        Allow the user to supply "bads" as a string for auto exclusion.
+
+    Returns
+    -------
+    chs : list
+        Channels to be excluded/excluded. If allow_bads, and chs=="bads",
+        this will be the bad channels found in 'info'.
+    """
+    from .meas_info import Info
+    if not isinstance(chs, (list, tuple, np.ndarray)):
+        if allow_bads is True:
+            if not isinstance(info, Info):
+                raise ValueError('Supply an info object if allow_bads is true')
+            elif chs != 'bads':
+                raise ValueError('If chs is a string, it must be "bads"')
+            else:
+                chs = info['bads']
+        else:
+            raise ValueError(
+                'include/exclude must be list, tuple, ndarray, or "bads". ' +
+                'You provided type {0}'.format(type(chs)))
+    return chs
