@@ -11,6 +11,9 @@ from .maxfilter import apply_maxfilter
 from .ssp import compute_proj_ecg, compute_proj_eog
 from .eog import find_eog_events, create_eog_epochs
 from .ecg import find_ecg_events, create_ecg_epochs
-from .ica import (ICA, ica_find_eog_events, ica_find_ecg_events, score_funcs,
-                  read_ica, run_ica)
+from .ica import (ICA, ica_find_eog_events, ica_find_ecg_events,
+                  get_score_funcs, read_ica, run_ica)
 from .bads import find_outliers
+from .stim import fix_stim_artifact
+from .maxwell import _maxwell_filter
+from .xdawn import Xdawn
