@@ -1,3 +1,10 @@
+# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+#          Denis Engemann <denis.engemann@gmail.com>
+#          Eric Larson <larson.eric.d@gmail.com>
+#
+# License: BSD (3-clause)
+
+import warnings
 from ..externals.six import string_types
 import numpy as np
 
@@ -82,8 +89,8 @@ def qrs_detector(sfreq, ecg, thresh_value=0.6, levels=2.5, n_thresh=3,
             if window[0] > thresh1:
                 max_time = np.argmax(window)
                 time.append(ii + max_time)
-                nx = np.sum(np.diff(((window > thresh1).astype(np.int)
-                                     == 1).astype(int)))
+                nx = np.sum(np.diff(((window > thresh1).astype(np.int) ==
+                                     1).astype(int)))
                 numcross.append(nx)
                 rms.append(np.sqrt(sum_squared(window) / window.size))
                 ii += win_size
@@ -105,8 +112,8 @@ def qrs_detector(sfreq, ecg, thresh_value=0.6, levels=2.5, n_thresh=3,
         clean_events.append(ce)
 
     # pick the best threshold; first get effective heart rates
-    rates = np.array([60. * len(ce) / (len(ecg) / float(sfreq))
-                      for ce in clean_events])
+    rates = np.array([60. * len(cev) / (len(ecg) / float(sfreq))
+                      for cev in clean_events])
 
     # now find heart rates that seem reasonable (infant thru adult athlete)
     idx = np.where(np.logical_and(rates <= 160., rates >= 40.))[0]
@@ -131,10 +138,11 @@ def find_ecg_events(raw, event_id=999, ch_name=None, tstart=0.0,
         The raw data
     event_id : int
         The index to assign to found events
-    ch_name : str
+    ch_name : None | str
         The name of the channel to use for ECG peak detection.
-        The argument is mandatory if the dataset contains no ECG
-        channels.
+        If None (default), a synthetic ECG channel is created from
+        cross channel average. Synthetic channel can only be created from
+        'meg' channels.
     tstart : float
         Start detection after tstart seconds. Useful when beginning
         of run is noisy.
@@ -160,16 +168,13 @@ def find_ecg_events(raw, event_id=999, ch_name=None, tstart=0.0,
     average_pulse : float
         Estimated average pulse.
     """
-    try:
-        idx_ecg = _get_ecg_channel_index(ch_name, raw)
-        assert len(idx_ecg) == 1
-        logger.info('Using channel %s to identify heart beats'
-                    % raw.ch_names[idx_ecg[0]])
-
+    idx_ecg = _get_ecg_channel_index(ch_name, raw)
+    if idx_ecg is not None:
+        logger.info('Using channel %s to identify heart beats.'
+                    % raw.ch_names[idx_ecg])
         ecg, times = raw[idx_ecg, :]
-    except RuntimeError:
+    else:
         ecg, times = _make_ecg(raw, None, None, verbose)
-        idx_ecg = None
 
     # detecting QRS and generating event file
     ecg_events = qrs_detector(raw.info['sfreq'], ecg.ravel(), tstart=tstart,
@@ -181,34 +186,40 @@ def find_ecg_events(raw, event_id=999, ch_name=None, tstart=0.0,
     logger.info("Number of ECG events detected : %d (average pulse %d / "
                 "min.)" % (n_events, average_pulse))
 
-    ecg_events = np.c_[ecg_events + raw.first_samp, np.zeros(n_events),
-                       event_id * np.ones(n_events)]
+    ecg_events = np.array([ecg_events + raw.first_samp,
+                           np.zeros(n_events, int),
+                           event_id * np.ones(n_events, int)]).T
     return ecg_events, idx_ecg, average_pulse
 
 
 def _get_ecg_channel_index(ch_name, inst):
-     # Geting ECG Channel
+    """Geting ECG channel index. If no channel found returns None."""
     if ch_name is None:
         ecg_idx = pick_types(inst.info, meg=False, eeg=False, stim=False,
                              eog=False, ecg=True, emg=False, ref_meg=False,
                              exclude='bads')
     else:
-        ecg_idx = pick_channels(inst.ch_names, include=[ch_name])
-        if len(ecg_idx) == 0:
+        if ch_name not in inst.ch_names:
             raise ValueError('%s not in channel list (%s)' %
                              (ch_name, inst.ch_names))
+        ecg_idx = pick_channels(inst.ch_names, include=[ch_name])
 
-    if len(ecg_idx) == 0 and ch_name is None:
-        raise RuntimeError('No ECG channel found. Please specify ch_name '
-                           'parameter e.g. MEG 1531')
+    if len(ecg_idx) == 0:
+        return None
+        # raise RuntimeError('No ECG channel found. Please specify ch_name '
+        #                    'parameter e.g. MEG 1531')
 
-    return ecg_idx
+    if len(ecg_idx) > 1:
+        warnings.warn('More than one ECG channel found. Using only %s.'
+                      % inst.ch_names[ecg_idx[0]])
+
+    return ecg_idx[0]
 
 
 @verbose
 def create_ecg_epochs(raw, ch_name=None, event_id=999, picks=None,
                       tmin=-0.5, tmax=0.5, l_freq=8, h_freq=16, reject=None,
-                      flat=None, verbose=None, baseline=None):
+                      flat=None, baseline=None, preload=True, verbose=None):
     """Conveniently generate epochs around ECG artifact events
 
 
@@ -216,10 +227,11 @@ def create_ecg_epochs(raw, ch_name=None, event_id=999, picks=None,
     ----------
     raw : instance of Raw
         The raw data
-    ch_name : str
+    ch_name : None | str
         The name of the channel to use for ECG peak detection.
-        The argument is mandatory if the dataset contains no ECG
-        channels.
+        If None (default), a synthetic ECG channel is created from
+        cross channel average. Synthetic channel can only be created from
+        'meg' channels.
     event_id : int
         The index to assign to found events
     picks : array-like of int | None (default)
@@ -233,14 +245,20 @@ def create_ecg_epochs(raw, ch_name=None, event_id=999, picks=None,
     h_freq : float
         High pass frequency.
     reject : dict | None
-        Rejection parameters based on peak to peak amplitude.
+        Rejection parameters based on peak-to-peak amplitude.
         Valid keys are 'grad' | 'mag' | 'eeg' | 'eog' | 'ecg'.
-        If reject is None then no rejection is done. You should
-        use such parameters to reject big measurement artifacts
-        and not ECG for example
+        If reject is None then no rejection is done. Example::
+
+            reject = dict(grad=4000e-13, # T / m (gradiometers)
+                          mag=4e-12, # T (magnetometers)
+                          eeg=40e-6, # uV (EEG channels)
+                          eog=250e-6 # uV (EOG channels)
+                          )
+
     flat : dict | None
-        Rejection parameters based on flatness of signal
-        Valid keys are 'grad' | 'mag' | 'eeg' | 'eog' | 'ecg'
+        Rejection parameters based on flatness of signal.
+        Valid keys are 'grad' | 'mag' | 'eeg' | 'eog' | 'ecg', and values
+        are floats that set the minimum acceptable peak-to-peak amplitude.
         If flat is None then no rejection is done.
     baseline : tuple or list of length 2, or None
         The time interval to apply rescaling / baseline correction.
@@ -250,26 +268,28 @@ def create_ecg_epochs(raw, ch_name=None, event_id=999, picks=None,
         and if b is None then b is set to the end of the interval.
         If baseline is equal ot (None, None) all the time
         interval is used. If None, no correction is applied.
+    preload : bool
+        Preload epochs or not.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see mne.verbose).
 
     Returns
     -------
-    eog_epochs : instance of Epochs
-        Data epoched around EOG events.
+    ecg_epochs : instance of Epochs
+        Data epoched around ECG r-peaks.
     """
 
     events, _, _ = find_ecg_events(raw, ch_name=ch_name, event_id=event_id,
                                    l_freq=l_freq, h_freq=h_freq,
                                    verbose=verbose)
-    if picks is not None:
+    if picks is None:
         picks = pick_types(raw.info, meg=True, eeg=True, ref_meg=False)
 
     # create epochs around ECG events and baseline (important)
     ecg_epochs = Epochs(raw, events=events, event_id=event_id,
                         tmin=tmin, tmax=tmax, proj=False,
                         picks=picks, reject=reject, baseline=baseline,
-                        verbose=verbose, preload=True)
+                        verbose=verbose, preload=preload)
     return ecg_epochs
 
 
@@ -277,7 +297,7 @@ def create_ecg_epochs(raw, ch_name=None, event_id=999, picks=None,
 def _make_ecg(inst, start, stop, verbose=None):
     """Create ECG signal from cross channel average
     """
-    if not any([c in inst for c in ['mag', 'grad']]):
+    if not any(c in inst for c in ['mag', 'grad']):
         raise ValueError('Unable to generate artifical ECG channel')
     for ch in ['mag', 'grad']:
         if ch in inst:
