@@ -1,6 +1,7 @@
 """Functions for statistical analysis"""
 
-from .parametric import f_threshold_twoway_rm, f_twoway_rm
+from .parametric import (
+    f_threshold_twoway_rm, f_threshold_mway_rm, f_twoway_rm, f_mway_rm)
 from .permutations import permutation_t_test
 from .cluster_level import (permutation_cluster_test,
                             permutation_cluster_1samp_test,
@@ -10,4 +11,4 @@ from .cluster_level import (permutation_cluster_test,
                             ttest_1samp_no_p,
                             summarize_clusters_stc)
 from .multi_comp import fdr_correction, bonferroni_correction
-from .regression import linear_regression
+from .regression import linear_regression, linear_regression_raw
