@@ -1,36 +1,28 @@
+from glob import glob
 import os
 
-from nose.tools import assert_raises, assert_true, assert_equal
+from nose.tools import assert_raises, assert_true
 import numpy as np
-from numpy.testing import (assert_array_equal, assert_array_almost_equal,
-                           assert_array_less)
+from numpy.testing import assert_array_almost_equal, assert_array_less
 
+import mne
 from mne.transforms import apply_trans, rotation, translation, scaling
 from mne.coreg import (fit_matched_points, fit_point_cloud,
                        _point_cloud_error, _decimate_points,
                        create_default_subject, scale_mri,
-                       _is_mri_subject, scale_labels, scale_source_space,
-                       read_elp)
-from mne.io.kit.tests import data_dir as kit_data_dir
-from mne.utils import requires_mne_fs_in_env, _TempDir, run_subprocess
+                       _is_mri_subject, scale_labels, scale_source_space)
+from mne.utils import (requires_mne, requires_freesurfer, _TempDir,
+                       run_tests_if_main, requires_version)
 from functools import reduce
 
 
-tempdir = _TempDir()
-
-
-def test_read_elp():
-    """Test reading an ELP file"""
-    path = os.path.join(kit_data_dir, 'test_elp.txt')
-    points = read_elp(path)
-    assert_equal(points.shape, (8, 3))
-    assert_array_equal(points[0], [1.3930, 13.1613, -4.6967])
-
-
-@requires_mne_fs_in_env
+@requires_mne
+@requires_freesurfer
+@requires_version('scipy', '0.11')
 def test_scale_mri():
     """Test creating fsaverage and scaling it"""
     # create fsaverage
+    tempdir = _TempDir()
     create_default_subject(subjects_dir=tempdir)
     is_mri = _is_mri_subject('fsaverage', tempdir)
     assert_true(is_mri, "Creating fsaverage failed")
@@ -41,29 +33,42 @@ def test_scale_mri():
     create_default_subject(update=True, subjects_dir=tempdir)
     assert_true(os.path.exists(fid_path), "Updating fsaverage")
 
+    # remove redundant label files
+    label_temp = os.path.join(tempdir, 'fsaverage', 'label', '*.label')
+    label_paths = glob(label_temp)
+    for label_path in label_paths[1:]:
+        os.remove(label_path)
+
     # create source space
-    path = os.path.join(tempdir, 'fsaverage', 'bem', 'fsaverage-ico-6-src.fif')
-    if not os.path.exists(path):
-        cmd = ['mne_setup_source_space', '--subject', 'fsaverage', '--ico',
-               '6']
-        env = os.environ.copy()
-        env['SUBJECTS_DIR'] = tempdir
-        run_subprocess(cmd, env=env)
+    path = os.path.join(tempdir, 'fsaverage', 'bem', 'fsaverage-ico-0-src.fif')
+    mne.setup_source_space('fsaverage', path, 'ico0', overwrite=True,
+                           subjects_dir=tempdir, add_dist=False)
 
     # scale fsaverage
-    scale_mri('fsaverage', 'flachkopf', [1, .2, .8], True, subjects_dir=tempdir)
+    os.environ['_MNE_FEW_SURFACES'] = 'true'
+    scale_mri('fsaverage', 'flachkopf', [1, .2, .8], True,
+              subjects_dir=tempdir)
+    del os.environ['_MNE_FEW_SURFACES']
     is_mri = _is_mri_subject('flachkopf', tempdir)
     assert_true(is_mri, "Scaling fsaverage failed")
     src_path = os.path.join(tempdir, 'flachkopf', 'bem',
-                            'flachkopf-ico-6-src.fif')
+                            'flachkopf-ico-0-src.fif')
     assert_true(os.path.exists(src_path), "Source space was not scaled")
     scale_labels('flachkopf', subjects_dir=tempdir)
 
     # scale source space separately
     os.remove(src_path)
-    scale_source_space('flachkopf', 'ico-6', subjects_dir=tempdir)
+    scale_source_space('flachkopf', 'ico-0', subjects_dir=tempdir)
     assert_true(os.path.exists(src_path), "Source space was not scaled")
 
+    # add distances to source space
+    src = mne.read_source_spaces(path)
+    mne.add_source_space_distances(src)
+    src.save(path)
+
+    # scale with distances
+    os.remove(src_path)
+    scale_source_space('flachkopf', 'ico-0', subjects_dir=tempdir)
 
 
 def test_fit_matched_points():
@@ -83,7 +88,7 @@ def test_fit_matched_points():
     trans = np.dot(rotation(2, 6, 3), scaling(.5, .5, .5))
     src_pts = apply_trans(trans, tgt_pts)
     trans_est = fit_matched_points(src_pts, tgt_pts, translate=False, scale=1,
-                                out='trans')
+                                   out='trans')
     est_pts = apply_trans(trans_est, src_pts)
     assert_array_almost_equal(tgt_pts, est_pts, 2, "fit_matched_points with "
                               "rotation and scaling.")
@@ -164,3 +169,6 @@ def test_fit_point_cloud():
     err = _point_cloud_error(est_pts, tgt_pts)
     assert_array_less(err, .1, "fit_point_cloud with rotation and 3 scaling "
                       "parameters.")
+
+
+run_tests_if_main()
