@@ -1,6 +1,7 @@
 # Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #         Denis Engemann <denis.engemann@gmail.com>
 #         Andrew Dykstra <andrew.r.dykstra@gmail.com>
+#         Mads Jensen <mje.mads@gmail.com>
 #
 # License: BSD (3-clause)
 
@@ -9,16 +10,17 @@ from copy import deepcopy
 import warnings
 
 import numpy as np
+from scipy import fftpack
 from numpy.testing import (assert_array_almost_equal, assert_equal,
                            assert_array_equal, assert_allclose)
 from nose.tools import assert_true, assert_raises, assert_not_equal
 
-from mne import (equalize_channels, pick_types, read_evoked, write_evoked,
-                 read_evokeds, write_evokeds)
+from mne import (equalize_channels, pick_types, read_evokeds, write_evokeds,
+                 grand_average, combine_evoked)
 from mne.evoked import _get_peak, EvokedArray
 from mne.epochs import EpochsArray
 
-from mne.utils import _TempDir, requires_pandas, requires_nitime
+from mne.utils import _TempDir, requires_pandas, slow_test, requires_version
 
 from mne.io.meas_info import create_info
 from mne.externals.six.moves import cPickle as pickle
@@ -30,7 +32,27 @@ fname = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data',
 fname_gz = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data',
                    'test-ave.fif.gz')
 
-tempdir = _TempDir()
+
+@requires_version('scipy', '0.14')
+def test_savgol_filter():
+    """Test savgol filtering
+    """
+    h_freq = 10.
+    evoked = read_evokeds(fname, 0)
+    freqs = fftpack.fftfreq(len(evoked.times), 1. / evoked.info['sfreq'])
+    data = np.abs(fftpack.fft(evoked.data))
+    match_mask = np.logical_and(freqs >= 0, freqs <= h_freq / 2.)
+    mismatch_mask = np.logical_and(freqs >= h_freq * 2, freqs < 50.)
+    assert_raises(ValueError, evoked.savgol_filter, evoked.info['sfreq'])
+    evoked.savgol_filter(h_freq)
+    data_filt = np.abs(fftpack.fft(evoked.data))
+    # decent in pass-band
+    assert_allclose(np.mean(data[:, match_mask], 0),
+                    np.mean(data_filt[:, match_mask], 0),
+                    rtol=1e-4, atol=1e-2)
+    # suppression in stop-band
+    assert_true(np.mean(data[:, mismatch_mask]) >
+                np.mean(data_filt[:, mismatch_mask]) * 5)
 
 
 def test_hash_evoked():
@@ -46,9 +68,11 @@ def test_hash_evoked():
     assert_not_equal(hash(ave), hash(ave_2))
 
 
+@slow_test
 def test_io_evoked():
     """Test IO for evoked data (fif + gz) with integer and str args
     """
+    tempdir = _TempDir()
     ave = read_evokeds(fname, 0)
 
     write_evokeds(op.join(tempdir, 'evoked-ave.fif'), ave)
@@ -62,6 +86,7 @@ def test_io_evoked():
     assert_equal(ave.kind, ave2.kind)
     assert_equal(ave.last, ave2.last)
     assert_equal(ave.first, ave2.first)
+    assert_true(repr(ave))
 
     # test compressed i/o
     ave2 = read_evokeds(fname_gz, 0)
@@ -75,17 +100,6 @@ def test_io_evoked():
     ave3 = read_evokeds(fname, condition)
     assert_array_almost_equal(ave.data, ave3.data, 19)
 
-    # test deprecation warning for read_evoked and write_evoked
-    # XXX should be deleted for 0.9 release
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter('always')
-        ave = read_evoked(fname, setno=0)
-        assert_true(w[0].category == DeprecationWarning)
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter('always')
-        write_evoked(op.join(tempdir, 'evoked-ave.fif'), ave)
-        assert_true(w[0].category == DeprecationWarning)
-
     # test read_evokeds and write_evokeds
     types = ['Left Auditory', 'Right Auditory', 'Left visual', 'Right visual']
     aves1 = read_evokeds(fname)
@@ -116,6 +130,7 @@ def test_io_evoked():
 def test_shift_time_evoked():
     """ Test for shifting of time scale
     """
+    tempdir = _TempDir()
     # Shift backward
     ave = read_evokeds(fname, 0)
     ave.shift_time(-0.1, relative=True)
@@ -156,6 +171,7 @@ def test_shift_time_evoked():
 def test_evoked_resample():
     """Test for resampling of evoked data
     """
+    tempdir = _TempDir()
     # upsample, write it out, read it in
     ave = read_evokeds(fname, 0)
     sfreq_normal = ave.info['sfreq']
@@ -196,27 +212,14 @@ def test_evoked_detrend():
                             rtol=1e-8, atol=1e-16))
 
 
-@requires_nitime
-def test_evoked_to_nitime():
-    """ Test to_nitime """
-    ave = read_evokeds(fname, 0)
-    evoked_ts = ave.to_nitime()
-    assert_equal(evoked_ts.data, ave.data)
-
-    picks2 = [1, 2]
-    ave = read_evokeds(fname, 0)
-    evoked_ts = ave.to_nitime(picks=picks2)
-    assert_equal(evoked_ts.data, ave.data[picks2])
-
-
 @requires_pandas
-def test_as_data_frame():
+def test_to_data_frame():
     """Test evoked Pandas exporter"""
     ave = read_evokeds(fname, 0)
-    assert_raises(ValueError, ave.as_data_frame, picks=np.arange(400))
-    df = ave.as_data_frame()
+    assert_raises(ValueError, ave.to_data_frame, picks=np.arange(400))
+    df = ave.to_data_frame()
     assert_true((df.columns == ave.ch_names).all())
-    df = ave.as_data_frame(use_time_index=False)
+    df = ave.to_data_frame(index=None).reset_index('time')
     assert_true('time' in df.columns)
     assert_array_equal(df.values[:, 1], ave.data[0] * 1e13)
     assert_array_equal(df.values[:, 3], ave.data[2] * 1e15)
@@ -326,6 +329,14 @@ def test_pick_channels_mixin():
     assert_equal(ch_names, evoked.ch_names)
     assert_equal(len(ch_names), len(evoked.data))
 
+    evoked = read_evokeds(fname, condition=0, proj=True)
+    assert_true('meg' in evoked)
+    assert_true('eeg' in evoked)
+    evoked.pick_types(meg=False, eeg=True)
+    assert_true('meg' not in evoked)
+    assert_true('eeg' in evoked)
+    assert_true(len(evoked.ch_names) == 60)
+
 
 def test_equalize_channels():
     """Test equalization of channels
@@ -341,9 +352,67 @@ def test_equalize_channels():
         assert_equal(ch_names, e.ch_names)
 
 
+def test_evoked_arithmetic():
+    """Test evoked arithmetic
+    """
+    ev = read_evokeds(fname, condition=0)
+    ev1 = EvokedArray(np.ones_like(ev.data), ev.info, ev.times[0], nave=20)
+    ev2 = EvokedArray(-np.ones_like(ev.data), ev.info, ev.times[0], nave=10)
+
+    # combine_evoked([ev1, ev2]) should be the same as ev1 + ev2:
+    # data should be added according to their `nave` weights
+    # nave = ev1.nave + ev2.nave
+    ev = ev1 + ev2
+    assert_equal(ev.nave, ev1.nave + ev2.nave)
+    assert_allclose(ev.data, 1. / 3. * np.ones_like(ev.data))
+    ev = ev1 - ev2
+    assert_equal(ev.nave, ev1.nave + ev2.nave)
+    assert_equal(ev.comment, ev1.comment + ' - ' + ev2.comment)
+    assert_allclose(ev.data, np.ones_like(ev1.data))
+
+    # default comment behavior if evoked.comment is None
+    old_comment1 = ev1.comment
+    old_comment2 = ev2.comment
+    ev1.comment = None
+    with warnings.catch_warnings(record=True):
+        warnings.simplefilter('always')
+        ev = ev1 - ev2
+        assert_equal(ev.comment, 'unknown')
+    ev1.comment = old_comment1
+    ev2.comment = old_comment2
+
+    # equal weighting
+    ev = combine_evoked([ev1, ev2], weights='equal')
+    assert_allclose(ev.data, np.zeros_like(ev1.data))
+
+    # combine_evoked([ev1, ev2], weights=[1, 0]) should yield the same as ev1
+    ev = combine_evoked([ev1, ev2], weights=[1, 0])
+    assert_equal(ev.nave, ev1.nave)
+    assert_allclose(ev.data, ev1.data)
+
+    # simple subtraction (like in oddball)
+    ev = combine_evoked([ev1, ev2], weights=[1, -1])
+    assert_allclose(ev.data, 2 * np.ones_like(ev1.data))
+
+    assert_raises(ValueError, combine_evoked, [ev1, ev2], weights='foo')
+    assert_raises(ValueError, combine_evoked, [ev1, ev2], weights=[1])
+
+    # grand average
+    evoked1, evoked2 = read_evokeds(fname, condition=[0, 1], proj=True)
+    ch_names = evoked1.ch_names[2:]
+    evoked1.info['bads'] = ['EEG 008']  # test interpolation
+    evoked1.drop_channels(evoked1.ch_names[:1])
+    evoked2.drop_channels(evoked2.ch_names[1:2])
+    gave = grand_average([evoked1, evoked2])
+    assert_equal(gave.data.shape, [len(ch_names), evoked1.data.shape[1]])
+    assert_equal(ch_names, gave.ch_names)
+    assert_equal(gave.nave, 2)
+
+
 def test_array_epochs():
     """Test creating evoked from array
     """
+    tempdir = _TempDir()
 
     # creating
     rng = np.random.RandomState(42)
@@ -382,3 +451,33 @@ def test_array_epochs():
     types = ['eeg'] * 19
     info = create_info(ch_names, sfreq, types)
     assert_raises(ValueError, EvokedArray, data1, info, tmin=-0.01)
+
+
+def test_add_channels():
+    """Test evoked splitting / re-appending channel types
+    """
+    evoked = read_evokeds(fname, condition=0)
+    evoked.info['buffer_size_sec'] = None
+    evoked_eeg = evoked.pick_types(meg=False, eeg=True, copy=True)
+    evoked_meg = evoked.pick_types(meg=True, copy=True)
+    evoked_stim = evoked.pick_types(meg=False, stim=True, copy=True)
+    evoked_eeg_meg = evoked.pick_types(meg=True, eeg=True, copy=True)
+    evoked_new = evoked_meg.add_channels([evoked_eeg, evoked_stim], copy=True)
+    assert_true(all(ch in evoked_new.ch_names
+                    for ch in evoked_stim.ch_names + evoked_meg.ch_names))
+    evoked_new = evoked_meg.add_channels([evoked_eeg], copy=True)
+
+    assert_true(ch in evoked_new.ch_names for ch in evoked.ch_names)
+    assert_array_equal(evoked_new.data, evoked_eeg_meg.data)
+    assert_true(all(ch not in evoked_new.ch_names
+                    for ch in evoked_stim.ch_names))
+
+    # Now test errors
+    evoked_badsf = evoked_eeg.copy()
+    evoked_badsf.info['sfreq'] = 3.1415927
+    evoked_eeg = evoked_eeg.crop(-.1, .1)
+
+    assert_raises(RuntimeError, evoked_meg.add_channels, [evoked_badsf])
+    assert_raises(AssertionError, evoked_meg.add_channels, [evoked_eeg])
+    assert_raises(ValueError, evoked_meg.add_channels, [evoked_meg])
+    assert_raises(AssertionError, evoked_meg.add_channels, evoked_badsf)
