@@ -10,39 +10,46 @@ from numpy.testing import (assert_array_almost_equal, assert_array_equal,
 
 from scipy.fftpack import fft
 
-from mne.datasets import sample
+from mne.datasets import testing
 from mne import (stats, SourceEstimate, VolSourceEstimate, Label,
-                 read_source_spaces)
+                 read_source_spaces, MixedSourceEstimate)
 from mne import read_source_estimate, morph_data, extract_label_time_course
 from mne.source_estimate import (spatio_temporal_tris_connectivity,
                                  spatio_temporal_src_connectivity,
-                                 compute_morph_matrix, grade_to_vertices)
+                                 compute_morph_matrix, grade_to_vertices,
+                                 grade_to_tris)
 
 from mne.minimum_norm import read_inverse_operator
 from mne.label import read_labels_from_annot, label_sign_flip
 from mne.utils import (_TempDir, requires_pandas, requires_sklearn,
-                       requires_pytables)
+                       requires_h5py, run_tests_if_main, slow_test)
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
-data_path = sample.data_path(download=False)
+data_path = testing.data_path(download=False)
 subjects_dir = op.join(data_path, 'subjects')
-fname = op.join(data_path, 'MEG', 'sample', 'sample_audvis-meg-lh.stc')
 fname_inv = op.join(data_path, 'MEG', 'sample',
-                    'sample_audvis-meg-oct-6-meg-inv.fif')
-fname_vol = op.join(data_path, 'MEG', 'sample',
-                    'sample_audvis-grad-vol-7-fwd-sensmap-vol.w')
-fname_vsrc = op.join(data_path, 'MEG', 'sample',
-                     'sample_audvis-meg-vol-7-fwd.fif')
+                    'sample_audvis_trunc-meg-eeg-oct-6-meg-inv.fif')
 fname_t1 = op.join(data_path, 'subjects', 'sample', 'mri', 'T1.mgz')
-
-tempdir = _TempDir()
+fname_src = op.join(data_path, 'MEG', 'sample',
+                    'sample_audvis_trunc-meg-eeg-oct-6-fwd.fif')
+fname_stc = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc-meg')
+fname_smorph = op.join(data_path, 'MEG', 'sample',
+                       'sample_audvis_trunc-meg')
+fname_fmorph = op.join(data_path, 'MEG', 'sample',
+                       'fsaverage_audvis_trunc-meg')
+fname_vol = op.join(data_path, 'MEG', 'sample',
+                    'sample_audvis_trunc-grad-vol-7-fwd-sensmap-vol.w')
+fname_vsrc = op.join(data_path, 'MEG', 'sample',
+                     'sample_audvis_trunc-meg-vol-7-fwd.fif')
 
 
-@sample.requires_sample_data
+@slow_test
+@testing.requires_testing_data
 def test_volume_stc():
     """Test volume STCs
     """
+    tempdir = _TempDir()
     N = 100
     data = np.arange(N)[:, np.newaxis]
     datas = [data, data, np.arange(2)[:, np.newaxis]]
@@ -57,7 +64,7 @@ def test_volume_stc():
             stc_new.save(fname_temp)
             stc_new = read_source_estimate(fname_temp)
             assert_true(isinstance(stc_new, VolSourceEstimate))
-            assert_array_equal(vertno_read, stc_new.vertno)
+            assert_array_equal(vertno_read, stc_new.vertices)
             assert_array_almost_equal(stc.data, stc_new.data)
 
     # now let's actually read a MNE-C processed file
@@ -72,7 +79,7 @@ def test_volume_stc():
         stc_new.save(fname_temp, ftype='w')
         stc_new = read_source_estimate(fname_temp)
         assert_true(isinstance(stc_new, VolSourceEstimate))
-        assert_array_equal(stc.vertno, stc_new.vertno)
+        assert_array_equal(stc.vertices, stc_new.vertices)
         assert_array_almost_equal(stc.data, stc_new.data)
 
     # save the stc as a nifti file and export
@@ -108,19 +115,22 @@ def test_volume_stc():
         print('Save as nifti test skipped, needs NiBabel')
 
 
-@sample.requires_sample_data
+@testing.requires_testing_data
 def test_expand():
     """Test stc expansion
     """
-    stc = read_source_estimate(fname, 'sample')
+    stc = read_source_estimate(fname_stc, 'sample')
     assert_true('sample' in repr(stc))
     labels_lh = read_labels_from_annot('sample', 'aparc', 'lh',
                                        subjects_dir=subjects_dir)
-    stc_limited = stc.in_label(labels_lh[0] + labels_lh[1])
+    new_label = labels_lh[0] + labels_lh[1]
+    stc_limited = stc.in_label(new_label)
     stc_new = stc_limited.copy()
     stc_new.data.fill(0)
     for label in labels_lh[:2]:
-        stc_new += stc.in_label(label).expand(stc_limited.vertno)
+        stc_new += stc.in_label(label).expand(stc_limited.vertices)
+    assert_raises(TypeError, stc_new.expand, stc_limited.vertices[0])
+    assert_raises(ValueError, stc_new.expand, [stc_limited.vertices[0]])
     # make sure we can't add unless vertno agree
     assert_raises(ValueError, stc.__add__, stc.in_label(labels_lh[0]))
 
@@ -133,22 +143,24 @@ def _fake_stc(n_time=10):
 def test_io_stc():
     """Test IO for STC files
     """
+    tempdir = _TempDir()
     stc = _fake_stc()
     stc.save(op.join(tempdir, "tmp.stc"))
     stc2 = read_source_estimate(op.join(tempdir, "tmp.stc"))
 
     assert_array_almost_equal(stc.data, stc2.data)
     assert_array_almost_equal(stc.tmin, stc2.tmin)
-    assert_equal(len(stc.vertno), len(stc2.vertno))
-    for v1, v2 in zip(stc.vertno, stc2.vertno):
+    assert_equal(len(stc.vertices), len(stc2.vertices))
+    for v1, v2 in zip(stc.vertices, stc2.vertices):
         assert_array_almost_equal(v1, v2)
     assert_array_almost_equal(stc.tstep, stc2.tstep)
 
 
-@requires_pytables()
+@requires_h5py
 def test_io_stc_h5():
     """Test IO for STC files using HDF5
     """
+    tempdir = _TempDir()
     stc = _fake_stc()
     assert_raises(ValueError, stc.save, op.join(tempdir, 'tmp'), ftype='foo')
     out_name = op.join(tempdir, 'tmp')
@@ -161,14 +173,15 @@ def test_io_stc_h5():
         assert_array_equal(stc_new.data, stc.data)
         assert_array_equal(stc_new.tmin, stc.tmin)
         assert_array_equal(stc_new.tstep, stc.tstep)
-        assert_equal(len(stc_new.vertno), len(stc.vertno))
-        for v1, v2 in zip(stc_new.vertno, stc.vertno):
+        assert_equal(len(stc_new.vertices), len(stc.vertices))
+        for v1, v2 in zip(stc_new.vertices, stc.vertices):
             assert_array_equal(v1, v2)
 
 
 def test_io_w():
     """Test IO for w files
     """
+    tempdir = _TempDir()
     stc = _fake_stc(n_time=1)
     w_fname = op.join(tempdir, 'fake')
     stc.save(w_fname, ftype='w')
@@ -201,7 +214,13 @@ def test_stc_arithmetic():
         a -= 1
         a *= -1
         a /= 2
-        a **= 3
+        b = 2 + a
+        b = 2 - a
+        b = +a
+        assert_array_equal(b.data, a.data)
+        with warnings.catch_warnings(record=True):
+            warnings.simplefilter('always')
+            a **= 3
         out.append(a)
 
     assert_array_equal(out[0], out[1].data)
@@ -211,12 +230,12 @@ def test_stc_arithmetic():
     assert_array_equal(stc_mean.data, np.mean(stc.data, 1)[:, None])
 
 
-@sample.requires_sample_data
+@slow_test
+@testing.requires_testing_data
 def test_stc_methods():
     """Test stc methods lh_data, rh_data, bin(), center_of_mass(), resample()
     """
-    fname = op.join(data_path, 'MEG', 'sample', 'sample_audvis-meg')
-    stc = read_source_estimate(fname)
+    stc = read_source_estimate(fname_stc)
 
     # lh_data / rh_data
     assert_array_equal(stc.lh_data, stc.data[:len(stc.lh_vertno)])
@@ -233,10 +252,31 @@ def test_stc_methods():
     vertex, hemi, t = stc.center_of_mass('sample', subjects_dir=subjects_dir)
     assert_true(hemi == 1)
     # XXX Should design a fool-proof test case, but here were the results:
-    assert_true(vertex == 90186)
-    assert_true(np.round(t, 3) == 0.123)
+    assert_equal(vertex, 124791)
+    assert_equal(np.round(t, 2), 0.12)
+
+    stc = read_source_estimate(fname_stc)
+    stc.subject = 'sample'
+    label_lh = read_labels_from_annot('sample', 'aparc', 'lh',
+                                      subjects_dir=subjects_dir)[0]
+    label_rh = read_labels_from_annot('sample', 'aparc', 'rh',
+                                      subjects_dir=subjects_dir)[0]
+    label_both = label_lh + label_rh
+    for label in (label_lh, label_rh, label_both):
+        assert_true(isinstance(stc.shape, tuple) and len(stc.shape) == 2)
+        stc_label = stc.in_label(label)
+        if label.hemi != 'both':
+            if label.hemi == 'lh':
+                verts = stc_label.vertices[0]
+            else:  # label.hemi == 'rh':
+                verts = stc_label.vertices[1]
+            n_vertices_used = len(label.get_vertices_used(verts))
+            assert_equal(len(stc_label.data), n_vertices_used)
+    stc_lh = stc.in_label(label_lh)
+    assert_raises(ValueError, stc_lh.in_label, label_rh)
+    label_lh.subject = 'foo'
+    assert_raises(RuntimeError, stc.in_label, label_lh)
 
-    stc = read_source_estimate(fname)
     stc_new = deepcopy(stc)
     o_sfreq = 1.0 / stc.tstep
     # note that using no padding for this STC reduces edge ringing...
@@ -249,7 +289,7 @@ def test_stc_methods():
     assert_array_almost_equal(stc_new.data, stc.data, 5)
 
 
-@sample.requires_sample_data
+@testing.requires_testing_data
 def test_extract_label_time_course():
     """Test extraction of label time courses from stc
     """
@@ -342,19 +382,21 @@ def test_extract_label_time_course():
     assert_true(x.size == 0)
 
 
-@sample.requires_sample_data
+@slow_test
+@testing.requires_testing_data
 def test_morph_data():
     """Test morphing of data
     """
+    tempdir = _TempDir()
     subject_from = 'sample'
     subject_to = 'fsaverage'
-    fname = op.join(data_path, 'MEG', 'sample', 'sample_audvis-meg')
-    stc_from = read_source_estimate(fname, subject='sample')
-    fname = op.join(data_path, 'MEG', 'sample', 'fsaverage_audvis-meg')
-    stc_to = read_source_estimate(fname)
+    stc_from = read_source_estimate(fname_smorph, subject='sample')
+    stc_to = read_source_estimate(fname_fmorph)
     # make sure we can specify grade
     stc_from.crop(0.09, 0.1)  # for faster computation
     stc_to.crop(0.09, 0.1)  # for faster computation
+    assert_raises(ValueError, stc_from.morph, subject_to, grade=3, smooth=-1,
+                  subjects_dir=subjects_dir)
     stc_to1 = stc_from.morph(subject_to, grade=3, smooth=12, buffer_size=1000,
                              subjects_dir=subjects_dir)
     stc_to1.save(op.join(tempdir, '%s_audvis-meg' % subject_to))
@@ -368,16 +410,39 @@ def test_morph_data():
     stc_to3 = morph_data(subject_from, subject_to, stc_from,
                          grade=vertices_to, smooth=12, buffer_size=3,
                          subjects_dir=subjects_dir)
+    # make sure we get a warning about # of steps
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        morph_data(subject_from, subject_to, stc_from,
+                   grade=vertices_to, smooth=1, buffer_size=3,
+                   subjects_dir=subjects_dir)
+    assert_equal(len(w), 2)
 
     assert_array_almost_equal(stc_to.data, stc_to1.data, 5)
     assert_array_almost_equal(stc_to1.data, stc_to2.data)
     assert_array_almost_equal(stc_to1.data, stc_to3.data)
     # make sure precomputed morph matrices work
     morph_mat = compute_morph_matrix(subject_from, subject_to,
-                                     stc_from.vertno, vertices_to,
+                                     stc_from.vertices, vertices_to,
                                      smooth=12, subjects_dir=subjects_dir)
     stc_to3 = stc_from.morph_precomputed(subject_to, vertices_to, morph_mat)
     assert_array_almost_equal(stc_to1.data, stc_to3.data)
+    assert_raises(ValueError, stc_from.morph_precomputed,
+                  subject_to, vertices_to, 'foo')
+    assert_raises(ValueError, stc_from.morph_precomputed,
+                  subject_to, [vertices_to[0]], morph_mat)
+    assert_raises(ValueError, stc_from.morph_precomputed,
+                  subject_to, [vertices_to[0][:-1], vertices_to[1]], morph_mat)
+    assert_raises(ValueError, stc_from.morph_precomputed, subject_to,
+                  vertices_to, morph_mat, subject_from='foo')
+
+    # steps warning
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        compute_morph_matrix(subject_from, subject_to,
+                             stc_from.vertices, vertices_to,
+                             smooth=1, subjects_dir=subjects_dir)
+    assert_equal(len(w), 2)
 
     mean_from = stc_from.data.mean(axis=0)
     mean_to = stc_to1.data.mean(axis=0)
@@ -388,19 +453,10 @@ def test_morph_data():
                          smooth=12, buffer_size=3, subjects_dir=subjects_dir)
     assert_true(stc_to5.data.shape[0] == 163842 + 163842)
 
-    # test morphing to the same subject
-    stc_to6 = stc_from.morph(subject_from, grade=stc_from.vertno, smooth=1,
-                             subjects_dir=subjects_dir)
-    mask = np.ones(stc_from.data.shape[0], dtype=np.bool)
-    # XXX: there is a bug somewhere that causes a difference at 2 vertices..
-    mask[6799] = False
-    mask[6800] = False
-    assert_array_almost_equal(stc_from.data[mask], stc_to6.data[mask], 5)
-
     # Morph sparse data
     # Make a sparse stc
-    stc_from.vertno[0] = stc_from.vertno[0][[100, 500]]
-    stc_from.vertno[1] = stc_from.vertno[1][[200]]
+    stc_from.vertices[0] = stc_from.vertices[0][[100, 500]]
+    stc_from.vertices[1] = stc_from.vertices[1][[200]]
     stc_from._data = stc_from._data[:3]
 
     assert_raises(RuntimeError, stc_from.morph, subject_to, sparse=True,
@@ -416,7 +472,7 @@ def test_morph_data():
     assert_equal(stc_from.tmin, stc_from.tmin)
     assert_equal(stc_from.tstep, stc_from.tstep)
 
-    stc_from.vertno[0] = np.array([], dtype=np.int64)
+    stc_from.vertices[0] = np.array([], dtype=np.int64)
     stc_from._data = stc_from._data[:1]
 
     stc_to_sparse = stc_from.morph(subject_to, grade=None, sparse=True,
@@ -479,7 +535,7 @@ def test_transform():
     stcs_t = stc.transform(_my_trans, copy=True)
     assert_true(isinstance(stcs_t, list))
     assert_array_equal(stc.times, stcs_t[0].times)
-    assert_equal(stc.vertno, stcs_t[0].vertno)
+    assert_equal(stc.vertices, stcs_t[0].vertices)
 
     data = np.concatenate((stcs_t[0].data[:, :, None],
                            stcs_t[1].data[:, :, None]), axis=2)
@@ -506,8 +562,8 @@ def test_transform():
     stc.transform(np.abs, idx=verts, tmin=-50, tmax=500, copy=False)
     assert_true(isinstance(stc, SourceEstimate))
     assert_true((stc.tmin == 0.) & (stc.times[-1] == 0.5))
-    assert_true(len(stc.vertno[0]) == 0)
-    assert_equal(stc.vertno[1], verts_rh)
+    assert_true(len(stc.vertices[0]) == 0)
+    assert_equal(stc.vertices[1], verts_rh)
     assert_array_equal(stc.data, data_t)
 
     times = np.round(1000 * stc.times)
@@ -535,7 +591,7 @@ def test_spatio_temporal_tris_connectivity():
         assert_array_equal(c, n)
 
 
-@sample.requires_sample_data
+@testing.requires_testing_data
 def test_spatio_temporal_src_connectivity():
     """Test spatio-temporal connectivity from source spaces"""
     tris = np.array([[0, 1, 2], [3, 4, 5]])
@@ -565,9 +621,11 @@ def test_spatio_temporal_src_connectivity():
     b = sum([s['nuse'] for s in inverse_operator['src']])
     assert_true(a == b)
 
+    assert_equal(grade_to_tris(5).shape, [40960, 3])
+
 
 @requires_pandas
-def test_as_data_frame():
+def test_to_data_frame():
     """Test stc Pandas exporter"""
     n_vert, n_times = 10, 5
     vertices = [np.arange(n_vert, dtype=np.int), np.empty(0, dtype=np.int)]
@@ -577,15 +635,15 @@ def test_as_data_frame():
     stc_vol = VolSourceEstimate(data, vertices=vertices[0], tmin=0, tstep=1,
                                 subject='sample')
     for stc in [stc_surf, stc_vol]:
-        assert_raises(ValueError, stc.as_data_frame, index=['foo', 'bar'])
+        assert_raises(ValueError, stc.to_data_frame, index=['foo', 'bar'])
         for ncat, ind in zip([1, 0], ['time', ['subject', 'time']]):
-            df = stc.as_data_frame(index=ind)
+            df = stc.to_data_frame(index=ind)
             assert_true(df.index.names == ind
                         if isinstance(ind, list) else [ind])
             assert_array_equal(df.values.T[ncat:], stc.data)
             # test that non-indexed data were present as categorial variables
-            with warnings.catch_warnings(record=True):  # pandas
-                df.reset_index().columns[:3] == ['subject', 'time']
+            assert_true(all([c in ['time', 'subject'] for c in
+                             df.reset_index().columns][:2]))
 
 
 def test_get_peak():
@@ -606,7 +664,7 @@ def test_get_peak():
         assert_raises(ValueError, stc.get_peak, tmin=0.002, tmax=0.001)
 
         vert_idx, time_idx = stc.get_peak()
-        vertno = np.concatenate(stc.vertno) if ii == 0 else stc.vertno
+        vertno = np.concatenate(stc.vertices) if ii == 0 else stc.vertices
         assert_true(vert_idx in vertno)
         assert_true(time_idx in stc.times)
 
@@ -614,3 +672,29 @@ def test_get_peak():
                                         time_as_index=True)
         assert_true(vert_idx < stc.data.shape[0])
         assert_true(time_idx < len(stc.times))
+
+
+@testing.requires_testing_data
+def test_mixed_stc():
+    """Test source estimate from mixed source space
+    """
+    N = 90  # number of sources
+    T = 2  # number of time points
+    S = 3  # number of source spaces
+
+    data = np.random.randn(N, T)
+    vertno = S * [np.arange(N // S)]
+
+    # make sure error is raised if vertices are not a list of length >= 2
+    assert_raises(ValueError, MixedSourceEstimate, data=data,
+                  vertices=[np.arange(N)])
+
+    stc = MixedSourceEstimate(data, vertno, 0, 1)
+
+    vol = read_source_spaces(fname_vsrc)
+
+    # make sure error is raised for plotting surface with volume source
+    assert_raises(ValueError, stc.plot_surface, src=vol)
+
+
+run_tests_if_main()
