@@ -4,6 +4,7 @@
 # License: BSD (3-clause)
 
 import os
+from os import path as op
 import glob
 import numpy as np
 from numpy import sin, cos
@@ -13,7 +14,7 @@ from .io.constants import FIFF
 from .io.open import fiff_open
 from .io.tag import read_tag
 from .io.write import start_file, end_file, write_coord_trans
-from .utils import check_fname, logger
+from .utils import check_fname, logger, deprecated
 from .externals.six import string_types
 
 
@@ -25,33 +26,98 @@ als_ras_trans = np.array([[0, -1, 0, 0], [1, 0, 0, 0], [0, 0, 1, 0],
 als_ras_trans_mm = als_ras_trans * [0.001, 0.001, 0.001, 1]
 
 
+_str_to_frame = dict(meg=FIFF.FIFFV_COORD_DEVICE,
+                     mri=FIFF.FIFFV_COORD_MRI,
+                     mri_voxel=FIFF.FIFFV_MNE_COORD_MRI_VOXEL,
+                     head=FIFF.FIFFV_COORD_HEAD,
+                     mni_tal=FIFF.FIFFV_MNE_COORD_MNI_TAL,
+                     ras=FIFF.FIFFV_MNE_COORD_RAS,
+                     fs_tal=FIFF.FIFFV_MNE_COORD_FS_TAL,
+                     ctf_head=FIFF.FIFFV_MNE_COORD_CTF_HEAD,
+                     ctf_meg=FIFF.FIFFV_MNE_COORD_CTF_DEVICE)
+_frame_to_str = dict((val, key) for key, val in _str_to_frame.items())
+
+_verbose_frames = {FIFF.FIFFV_COORD_UNKNOWN: 'unknown',
+                   FIFF.FIFFV_COORD_DEVICE: 'MEG device',
+                   FIFF.FIFFV_COORD_ISOTRAK: 'isotrak',
+                   FIFF.FIFFV_COORD_HPI: 'hpi',
+                   FIFF.FIFFV_COORD_HEAD: 'head',
+                   FIFF.FIFFV_COORD_MRI: 'MRI (surface RAS)',
+                   FIFF.FIFFV_MNE_COORD_MRI_VOXEL: 'MRI voxel',
+                   FIFF.FIFFV_COORD_MRI_SLICE: 'MRI slice',
+                   FIFF.FIFFV_COORD_MRI_DISPLAY: 'MRI display',
+                   FIFF.FIFFV_MNE_COORD_CTF_DEVICE: 'CTF MEG device',
+                   FIFF.FIFFV_MNE_COORD_CTF_HEAD: 'CTF/4D/KIT head',
+                   FIFF.FIFFV_MNE_COORD_RAS: 'RAS (non-zero origin)',
+                   FIFF.FIFFV_MNE_COORD_MNI_TAL: 'MNI Talairach',
+                   FIFF.FIFFV_MNE_COORD_FS_TAL_GTZ: 'Talairach (MNI z > 0)',
+                   FIFF.FIFFV_MNE_COORD_FS_TAL_LTZ: 'Talairach (MNI z < 0)',
+                   -1: 'unknown'}
+
+
+def _to_const(cf):
+    """Helper to convert string or int coord frame into int"""
+    if isinstance(cf, string_types):
+        if cf not in _str_to_frame:
+            raise ValueError('Unknown cf %s' % cf)
+        cf = _str_to_frame[cf]
+    elif not isinstance(cf, int):
+        raise TypeError('cf must be str or int, not %s' % type(cf))
+    return cf
+
+
+class Transform(dict):
+    """A transform
+
+    Parameters
+    ----------
+    fro : str | int
+        The starting coordinate frame.
+    to : str | int
+        The ending coordinate frame.
+    trans : array-like, shape (4, 4)
+        The transformation matrix.
+    """
+    def __init__(self, fro, to, trans):
+        super(Transform, self).__init__()
+        # we could add some better sanity checks here
+        fro = _to_const(fro)
+        to = _to_const(to)
+        trans = np.asarray(trans, dtype=np.float64)
+        if trans.shape != (4, 4):
+            raise ValueError('Transformation must be shape (4, 4) not %s'
+                             % (trans.shape,))
+        self['from'] = fro
+        self['to'] = to
+        self['trans'] = trans
+
+    def __repr__(self):
+        return ('<Transform  |  %s->%s>\n%s'
+                % (_coord_frame_name(self['from']),
+                   _coord_frame_name(self['to']), self['trans']))
+
+    @property
+    def from_str(self):
+        return _coord_frame_name(self['from'])
+
+    @property
+    def to_str(self):
+        return _coord_frame_name(self['to'])
+
+
 def _coord_frame_name(cframe):
-    """Map integers to human-readable names"""
-    types = {FIFF.FIFFV_COORD_UNKNOWN: 'unknown',
-             FIFF.FIFFV_COORD_DEVICE: 'MEG device',
-             FIFF.FIFFV_COORD_ISOTRAK: 'isotrak',
-             FIFF.FIFFV_COORD_HPI: 'hpi',
-             FIFF.FIFFV_COORD_HEAD: 'head',
-             FIFF.FIFFV_COORD_MRI: 'MRI (surface RAS)',
-             FIFF.FIFFV_MNE_COORD_MRI_VOXEL: 'MRI voxel',
-             FIFF.FIFFV_COORD_MRI_SLICE: 'MRI slice',
-             FIFF.FIFFV_COORD_MRI_DISPLAY: 'MRI display',
-             FIFF.FIFFV_MNE_COORD_CTF_DEVICE: 'CTF MEG device',
-             FIFF.FIFFV_MNE_COORD_CTF_HEAD: 'CTF/4D/KIT head',
-             FIFF.FIFFV_MNE_COORD_RAS: 'RAS (non-zero origin)',
-             FIFF.FIFFV_MNE_COORD_MNI_TAL: 'MNI Talairach',
-             FIFF.FIFFV_MNE_COORD_FS_TAL_GTZ: 'Talairach (MNI z > 0)',
-             FIFF.FIFFV_MNE_COORD_FS_TAL_LTZ: 'Talairach (MNI z < 0)',
-             -1: 'unknown'}
-    return types.get(cframe, 'unknown')
+    """Map integers to human-readable (verbose) names"""
+    return _verbose_frames.get(int(cframe), 'unknown')
 
 
 def _print_coord_trans(t, prefix='Coordinate transformation: '):
     logger.info(prefix + '%s -> %s'
                 % (_coord_frame_name(t['from']), _coord_frame_name(t['to'])))
-    for tt in t['trans']:
-        logger.info('    % 8.6f % 8.6f % 8.6f    %7.2f mm' %
-                    (tt[0], tt[1], tt[2], 1000 * tt[3]))
+    for ti, tt in enumerate(t['trans']):
+        scale = 1000. if ti != 3 else 1.
+        text = ' mm' if ti != 3 else ''
+        logger.info('    % 8.6f % 8.6f % 8.6f    %7.2f%s' %
+                    (tt[0], tt[1], tt[2], scale * tt[3], text))
 
 
 def _find_trans(subject, subjects_dir=None):
@@ -77,7 +143,7 @@ def apply_trans(trans, pts, move=True):
 
     Parameters
     ----------
-    trans : array, shape = (4, 4)
+    trans : array, shape = (4, 4) | instance of Transform
         Transform matrix.
     pts : array, shape = (3,) | (n, 3)
         Array with coordinates for one or n points.
@@ -89,17 +155,15 @@ def apply_trans(trans, pts, move=True):
     transformed_pts : shape = (3,) | (n, 3)
         Transformed point(s).
     """
+    if isinstance(trans, dict):
+        trans = trans['trans']
     trans = np.asarray(trans)
     pts = np.asarray(pts)
     if pts.size == 0:
         return pts.copy()
 
     # apply rotation & scale
-    if pts.ndim == 1:
-        out_pts = np.dot(trans[:3, :3], pts)
-    else:
-        out_pts = np.dot(pts, trans[:3, :3].T)
-
+    out_pts = np.dot(pts, trans[:3, :3].T)
     # apply translation
     if move is True:
         transl = trans[:3, 3]
@@ -226,18 +290,83 @@ def translation(x=0, y=0, z=0):
     return m
 
 
-def _get_mri_head_t_from_trans_file(fname):
-    """Helper to convert "-trans.txt" to "-trans.fif" mri-type equivalent"""
-    # Read a Neuromag -> FreeSurfer transformation matrix
-    t = np.genfromtxt(fname)
-    if t.ndim != 2 or t.shape != (4, 4):
-        raise RuntimeError('File "%s" did not have 4x4 entries' % fname)
-    t = {'from': FIFF.FIFFV_COORD_HEAD, 'to': FIFF.FIFFV_COORD_MRI, 'trans': t}
-    return invert_transform(t)
+def _ensure_trans(trans, fro='mri', to='head'):
+    """Helper to ensure we have the proper transform"""
+    if isinstance(fro, string_types):
+        from_str = fro
+        from_const = _str_to_frame[fro]
+    else:
+        from_str = _frame_to_str[fro]
+        from_const = fro
+    del fro
+    if isinstance(to, string_types):
+        to_str = to
+        to_const = _str_to_frame[to]
+    else:
+        to_str = _frame_to_str[to]
+        to_const = to
+    del to
+    err_str = 'trans must go %s<->%s, provided' % (from_str, to_str)
+    if trans is None:
+        raise ValueError('%s None' % err_str)
+    if set([trans['from'], trans['to']]) != set([from_const, to_const]):
+        raise ValueError('%s trans is %s->%s' % (err_str,
+                                                 _frame_to_str[trans['from']],
+                                                 _frame_to_str[trans['to']]))
+    if trans['from'] != from_const:
+        trans = invert_transform(trans)
+    return trans
+
+
+def _get_mri_head_t(trans):
+    """Get mri_head_t (from=mri, to=head) from mri filename"""
+    if isinstance(trans, string_types):
+        if not op.isfile(trans):
+            raise IOError('trans file "%s" not found' % trans)
+        if op.splitext(trans)[1] in ['.fif', '.gz']:
+            mri_head_t = read_trans(trans)
+        else:
+            # convert "-trans.txt" to "-trans.fif" mri-type equivalent
+            t = np.genfromtxt(trans)
+            if t.ndim != 2 or t.shape != (4, 4):
+                raise RuntimeError('File "%s" did not have 4x4 entries'
+                                   % trans)
+            mri_head_t = Transform('head', 'mri', t)
+    elif isinstance(trans, dict):
+        mri_head_t = trans
+        trans = 'dict'
+    elif trans is None:
+        mri_head_t = Transform('head', 'mri', np.eye(4))
+        trans = 'identity'
+    else:
+        raise ValueError('trans type %s not known, must be str, dict, or None'
+                         % type(trans))
+    # it's usually a head->MRI transform, so we probably need to invert it
+    mri_head_t = _ensure_trans(mri_head_t, 'mri', 'head')
+    return mri_head_t, trans
 
 
 def combine_transforms(t_first, t_second, fro, to):
-    """Combine two transforms"""
+    """Combine two transforms
+
+    Parameters
+    ----------
+    t_first : dict
+        First transform.
+    t_second : dict
+        Second transform.
+    fro : int
+        From coordinate frame.
+    to : int
+        To coordinate frame.
+
+    Returns
+    -------
+    trans : dict
+        Combined transformation.
+    """
+    fro = _to_const(fro)
+    to = _to_const(to)
     if t_first['from'] != fro:
         raise RuntimeError('From mismatch: %s ("%s") != %s ("%s")'
                            % (t_first['from'],
@@ -254,8 +383,7 @@ def combine_transforms(t_first, t_second, fro, to):
                            % (t_second['to'],
                               _coord_frame_name(t_second['to']),
                               to, _coord_frame_name(to)))
-    return {'from': fro, 'to': to, 'trans': np.dot(t_second['trans'],
-                                                   t_first['trans'])}
+    return Transform(fro, to, np.dot(t_second['trans'], t_first['trans']))
 
 
 def read_trans(fname):
@@ -271,10 +399,10 @@ def read_trans(fname):
     trans : dict
         The transformation dictionary from the fif file.
 
-    Notes
-    -----
-    The trans dictionary has the following structure:
-    trans = {'from': int, 'to': int, 'trans': numpy.ndarray <4x4>}
+    See Also
+    --------
+    write_trans
+    Transform
     """
     fid, tree, directory = fiff_open(fname)
 
@@ -299,9 +427,12 @@ def write_trans(fname, trans):
         The name of the file, which should end in '-trans.fif'.
     trans : dict
         Trans file data, as returned by read_trans.
+
+    See Also
+    --------
+    read_trans
     """
     check_fname(fname, 'trans', ('-trans.fif', '-trans.fif.gz'))
-
     fid = start_file(fname)
     write_coord_trans(fid, trans)
     end_file(fid)
@@ -309,15 +440,18 @@ def write_trans(fname, trans):
 
 def invert_transform(trans):
     """Invert a transformation between coordinate systems
+
+    Parameters
+    ----------
+    trans : dict
+        Transform to invert.
+
+    Returns
+    -------
+    inv_trans : dict
+        Inverse transform.
     """
-    itrans = {'to': trans['from'], 'from': trans['to'],
-              'trans': linalg.inv(trans['trans'])}
-    return itrans
-
-
-_frame_dict = dict(meg=FIFF.FIFFV_COORD_DEVICE,
-                   mri=FIFF.FIFFV_COORD_MRI,
-                   head=FIFF.FIFFV_COORD_HEAD)
+    return Transform(trans['to'], trans['from'], linalg.inv(trans['trans']))
 
 
 def transform_surface_to(surf, dest, trans):
@@ -325,9 +459,9 @@ def transform_surface_to(surf, dest, trans):
 
     Parameters
     ----------
-    src : dict
+    surf : dict
         Surface.
-    orig: 'meg' | 'mri' | 'head' | int
+    dest : 'meg' | 'mri' | 'head' | int
         Destination coordinate system. Can be an integer for using
         FIFF types.
     trans : dict
@@ -339,25 +473,21 @@ def transform_surface_to(surf, dest, trans):
         Transformed source space. Data are modified in-place.
     """
     if isinstance(dest, string_types):
-        if dest not in _frame_dict:
+        if dest not in _str_to_frame:
             raise KeyError('dest must be one of %s, not "%s"'
-                           % [list(_frame_dict.keys()), dest])
-        dest = _frame_dict[dest]  # convert to integer
+                           % (list(_str_to_frame.keys()), dest))
+        dest = _str_to_frame[dest]  # convert to integer
     if surf['coord_frame'] == dest:
         return surf
 
-    if trans['to'] == surf['coord_frame'] and trans['from'] == dest:
-        trans = invert_transform(trans)
-    elif trans['from'] != surf['coord_frame'] or trans['to'] != dest:
-        raise ValueError('Cannot transform the source space using this '
-                         'coordinate transformation')
-
+    trans = _ensure_trans(trans, int(surf['coord_frame']), dest)
     surf['coord_frame'] = dest
-    surf['rr'] = apply_trans(trans['trans'], surf['rr'])
-    surf['nn'] = apply_trans(trans['trans'], surf['nn'], move=False)
+    surf['rr'] = apply_trans(trans, surf['rr'])
+    surf['nn'] = apply_trans(trans, surf['nn'], move=False)
     return surf
 
 
+@deprecated('transform_coordinates is deprecated and will be removed in v0.11')
 def transform_coordinates(filename, pos, orig, dest):
     """Transform coordinates between various MRI-related coordinate frames
 
@@ -366,6 +496,7 @@ def transform_coordinates(filename, pos, orig, dest):
     filename: string
         Name of a fif file containing the coordinate transformations
         This file can be conveniently created with mne_collect_transforms
+        or ``collect_transforms``.
     pos: array of shape N x 3
         array of locations to transform (in meters)
     orig: 'meg' | 'mri'
@@ -382,8 +513,8 @@ def transform_coordinates(filename, pos, orig, dest):
     trans_pos: array of shape N x 3
         The transformed locations
 
-    Example
-    -------
+    Examples
+    --------
     transform_coordinates('all-trans.fif', np.eye(3), 'meg', 'fs_tal')
     transform_coordinates('all-trans.fif', np.eye(3), 'mri', 'mni_tal')
     """
@@ -421,9 +552,9 @@ def transform_coordinates(filename, pos, orig, dest):
     #
     #   Check we have everything we need
     #
-    if ((orig == FIFF.FIFFV_COORD_HEAD and T0 is None) or (T1 is None)
-            or (T2 is None) or (dest == FIFF.FIFFV_MNE_COORD_FS_TAL and
-                                ((T3minus is None) or (T3minus is None)))):
+    if ((orig == FIFF.FIFFV_COORD_HEAD and T0 is None) or (T1 is None) or
+            (T2 is None) or (dest == FIFF.FIFFV_MNE_COORD_FS_TAL and
+                             ((T3minus is None) or (T3minus is None)))):
         raise ValueError('All required coordinate transforms not found')
 
     #
@@ -463,77 +594,96 @@ def transform_coordinates(filename, pos, orig, dest):
     return trans_pos
 
 
-# @verbose
-# def transform_meg_chs(chs, trans, verbose=None):
-#     """
-#     %
-#     % [res, count] = fiff_transform_meg_chs(chs,trans)
-#     %
-#     % Move to another coordinate system in MEG channel channel info
-#     % Count gives the number of channels transformed
-#     %
-#     % NOTE: Only the coil_trans field is modified by this routine, not
-#     % loc which remains to reflect the original data read from the fif file
-#     %
-#     %
-#
-#     XXX
-#     """
-#
-#     res = copy.deepcopy(chs)
-#
-#     count = 0
-#     t = trans['trans']
-#     for ch in res:
-#         if (ch['kind'] == FIFF.FIFFV_MEG_CH
-#                                     or ch['kind'] == FIFF.FIFFV_REF_MEG_CH):
-#             if (ch['coord_frame'] == trans['from']
-#                                             and ch['coil_trans'] is not None):
-#                 ch['coil_trans'] = np.dot(t, ch['coil_trans'])
-#                 ch['coord_frame'] = trans['to']
-#                 count += 1
-#
-#     if count > 0:
-#         logger.info('    %d MEG channel locations transformed' % count)
-#
-#     return res, count
+def get_ras_to_neuromag_trans(nasion, lpa, rpa):
+    """Construct a transformation matrix to the MNE head coordinate system
 
-# @verbose
-# def transform_eeg_chs(chs, trans, verbose=None):
-#     """
-#     %
-#     % [res, count] = fiff_transform_eeg_chs(chs,trans)
-#     %
-#     % Move to another coordinate system in EEG channel channel info
-#     % Count gives the number of channels transformed
-#     %
-#     % NOTE: Only the eeg_loc field is modified by this routine, not
-#     % loc which remains to reflect the original data read from the fif file
-#     %
-#
-#     XXX
-#     """
-#     res = copy.deepcopy(chs)
-#
-#     count = 0
-#     #
-#     #   Output unaugmented vectors from the transformation
-#     #
-#     t = trans['trans'][:3,:]
-#     for ch in res:
-#         if ch['kind'] == FIFF.FIFFV_EEG_CH:
-#             if (ch['coord_frame'] == trans['from']
-#                                             and ch['eeg_loc'] is not None):
-#                 #
-#                 # Transform the augmented EEG location vectors
-#                 #
-#                 for p in range(ch['eeg_loc'].shape[1]):
-#                     ch['eeg_loc'][:, p] = np.dot(t,
-#                                                 np.r_[ch['eeg_loc'][:,p], 1])
-#                 count += 1
-#                 ch['coord_frame'] = trans['to']
-#
-#     if count > 0:
-#         logger.info('    %d EEG electrode locations transformed\n' % count)
-#
-#     return res, count
+    Construct a transformation matrix from an arbitrary RAS coordinate system
+    to the MNE head coordinate system, in which the x axis passes through the
+    two preauricular points, and the y axis passes through the nasion and is
+    normal to the x axis. (see mne manual, pg. 97)
+
+    Parameters
+    ----------
+    nasion : array_like, shape (3,)
+        Nasion point coordinate.
+    lpa : array_like, shape (3,)
+        Left peri-auricular point coordinate.
+    rpa : array_like, shape (3,)
+        Right peri-auricular point coordinate.
+
+    Returns
+    -------
+    trans : numpy.array, shape = (4, 4)
+        Transformation matrix to MNE head space.
+    """
+    # check input args
+    nasion = np.asarray(nasion)
+    lpa = np.asarray(lpa)
+    rpa = np.asarray(rpa)
+    for pt in (nasion, lpa, rpa):
+        if pt.ndim != 1 or len(pt) != 3:
+            raise ValueError("Points have to be provided as one dimensional "
+                             "arrays of length 3.")
+
+    right = rpa - lpa
+    right_unit = right / linalg.norm(right)
+
+    origin = lpa + np.dot(nasion - lpa, right_unit) * right_unit
+
+    anterior = nasion - origin
+    anterior_unit = anterior / linalg.norm(anterior)
+
+    superior_unit = np.cross(right_unit, anterior_unit)
+
+    x, y, z = -origin
+    origin_trans = translation(x, y, z)
+
+    trans_l = np.vstack((right_unit, anterior_unit, superior_unit, [0, 0, 0]))
+    trans_r = np.reshape([0, 0, 0, 1], (4, 1))
+    rot_trans = np.hstack((trans_l, trans_r))
+
+    trans = np.dot(rot_trans, origin_trans)
+    return trans
+
+
+@deprecated('collect_transforms is deprecated and will be removed in v0.11')
+def collect_transforms(fname, xforms):
+    """Collect a set of transforms in a single FIFF file
+
+    Parameters
+    ----------
+    fname : str
+        Filename to save to.
+    xforms : list of dict
+        List of transformations.
+    """
+    check_fname(fname, 'trans', ('-trans.fif', '-trans.fif.gz'))
+    with start_file(fname) as fid:
+        for xform in xforms:
+            write_coord_trans(fid, xform)
+        end_file(fid)
+
+
+def _sphere_to_cartesian(theta, phi, r):
+    """Transform spherical coordinates to cartesian"""
+    z = r * np.sin(phi)
+    rcos_phi = r * np.cos(phi)
+    x = rcos_phi * np.cos(theta)
+    y = rcos_phi * np.sin(theta)
+    return x, y, z
+
+
+def _polar_to_cartesian(theta, r):
+    """Transform polar coordinates to cartesian"""
+    x = r * np.cos(theta)
+    y = r * np.sin(theta)
+    return x, y
+
+
+def _cartesian_to_sphere(x, y, z):
+    """Transform cartesian coordinates to spherical"""
+    hypotxy = np.hypot(x, y)
+    r = np.hypot(hypotxy, z)
+    elev = np.arctan2(z, hypotxy)
+    az = np.arctan2(y, x)
+    return az, elev, r
